/* @(#) sm_fort.h 1.4 3/26/98 15:46:34 */
// This crypto token library header is usually the only include that the C++
// application needs.  Directly included by the C++ app and the fortezza
// crypto token library.

#ifndef _SM_FORT_H_
#define _SM_FORT_H_

#include "sm_api.h"
#include "sm_common.h"
#include "cryptint.h"


// error codes specific to the Fortezza CTI
//
// all Fortezza CTI error codes begin with 91000

                                      
#define  FORT_MISSING_PARAMS    91000
#define  FORT_INVALID_PIN_SIZE  91001

class CSM_Fortezza : public CSM_Common
{
public:
   SM_RET_VAL SMTI_Sign  (CSM_Buffer *pData, // input
                          CSM_Buffer *pEncryptedDigest, // output
                          CSM_Buffer *pDigest); // output

   SM_RET_VAL SMTI_Verify(CSM_Buffer *pSignerPublicKey, // input
                          CSM_Alg    *pDigestAlg,  // input
                          CSM_Alg    *pSignatureAlg, // input
                          CSM_Buffer *pData, // input
                          CSM_Buffer *pSignature); // input

   SM_RET_VAL SMTI_Encrypt(CSM_Buffer *pData, // input
                           CSM_Buffer *pParameters, // input
                           CSM_Buffer *pEncrytpedData); // output

   SM_RET_VAL SMTI_Decrypt(
            CSM_Buffer *pParameters, // input (initialization vector)
            CSM_Buffer *pEncryptedData, // input (data to be decrypted)
            CSM_Buffer *pData); // output (decrypted data)

   SM_RET_VAL SMTI_DigestData(
            CSM_Buffer *pData, // input
            CSM_Buffer *pDigest); // output

   SM_RET_VAL CSM_Fortezza::SMTI_Random(
            CSM_Buffer *pSeed,
            CSM_Buffer *pRandom,
            SM_SIZE_T lLength);

   SM_RET_VAL SMTI_GenerateEMEK(CSM_Buffer *pRecipient,
                                CSM_Buffer *pParameters,
                                CSM_Buffer *pEMEK);

   SM_RET_VAL SMTI_ExtractMEK(CSM_Buffer *pOriginator,
                              CSM_Buffer *pParameters,
                              CSM_Buffer *pEMEK);

 
   // the following constructor also init's the session pointer
        // within pCSMIME (PL: verify)
        //
   CSM_Fortezza          (CSMIME        *pCSMIME,
                          char          *pszPin,
                          long           nSocket);

   // member access functions
private:
   CI_PIN m_pszPin;
   long   m_nSocket;
   long   m_nCertIndex;

   SM_RET_VAL Login(CSMIME *pCSMIME);
};

// These classes are for handling Fortezza card card operations.  Like 
// retrieving Personality Lists, Downloading Certificates, etc.. etc..
//
class CSM_FortPerList
{
   private:
      CI_PERSON *mp_persList;    // array of all slots on the card
      int       *mp_persIndex[]; // index to personalities only!
      int        m_currIndex;

   public:
      CSM_FortPerList();
      int Next();     // 0 == success, 1 == failure
      int GetSlot();  // >= 0 == sucess , otherwise failure
};


// These classes are for handling Fortezza specific stuff within
// X.509 objects.

class CSM_FortDSAParams
{
    public: 
       unsigned char       P[CI_P_SIZE];
       unsigned char       Q[CI_Q_SIZE];
       unsigned char       G[CI_G_SIZE];
   
       CSM_FortDSAParams();
       SM_RET_VAL DecodeParams(CSM_Buffer *pParams);
};
#endif // _SM_FORT_H_

// EOF sm_fort.h

