/* @(#) sm_internal.h 1.21 3/25/98 10:26:56 */

//////////////////////////////////////////////////////////////////////////
// sm_internal.h
//
// This file has classes that the application may use but are intended
// for a low-level general use.
//
//////////////////////////////////////////////////////////////////////////
#ifndef _SM_INTERNAL_H_
#define _SM_INTERNAL_H_

#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <fstream.h>
#include "sm_vdasnacc.h" // vda snacc include

//////////////////////////////////////////////////////////////////////////
// GENERAL PURPOSE DEFINES
//////////////////////////////////////////////////////////////////////////

#ifndef SM_RET_VAL
#define SM_RET_VAL long
#endif

#ifndef SM_SIZE_T
#define SM_SIZE_T size_t
#endif

//////////////////////////////////////////////////////////////////////////
// ADDRESS BOOK STUFF
//////////////////////////////////////////////////////////////////////////
#define SM_DEFAULT_MAB_DATA "./certs/mab_data.dat"
#define SM_CARD_SPYRUS "SPYRUS"
#define SM_CARD_LITRONIC "LITRONIC"
#define SM_CARD_GTC "GTC"

//////////////////////////////////////////////////////////////////////////
// ERROR CONSTANTS
//////////////////////////////////////////////////////////////////////////
#define SM_UNKNOWN_ERROR                9999
#define SM_NO_ERROR                     0
//////////////////////////////////////////////////////////////////////////
// HIGH LEVEL ERROR CONSTANTS 1000-1999
#define SM_MEMORY_ERROR                 1000
#define SM_FILEIO_ERROR                 1001
#define SM_NO_FILENAME                  1002
#define SM_MISSING_PARAM                1003
#define SM_MAB_ERROR                    1004
#define SM_NO_INSTANCES                 1005
#define SM_NO_SUPPORTING_INSTANCE       1006
#define SM_NO_CERT_SET                  1007
#define SM_BAD_PUBLIC_VALUE             1008
#define SM_ENCRYPTION_UNPREPARED        1009

//////////////////////////////////////////////////////////////////////////
// ASN.1 ERROR CONSTANTS 2000-2999
#define SM_CERT_DEC_ERROR               2000
#define SM_ENV_DATA_DEC_ERROR           2001

//////////////////////////////////////////////////////////////////////////
// CLASS DEFINITIONS
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
// CSM_List is the double-linked-list template class used in the SFL
template <class T> 
class CSM_List
{
public:
   CSM_List() { m_This = NULL; m_first = this; m_last = this; 
         m_next = NULL; m_prev = NULL; }
   CSM_List(T *t);
   ~CSM_List() { SelfDestruct(); }
   void SelfDestruct(void);
   void Append(T *t);
   void Append(CSM_List<T> *List);
   void DestroyList();
   SM_RET_VAL Length();
   CSM_List<T>* First();
   CSM_List<T>* Last();
   CSM_List<T>* Next();
   CSM_List<T>* Prev();
   T* Get();
   void Set(T *t);
private:
   T* m_This;
   CSM_List<T>* m_first;
   CSM_List<T>* m_last;
   CSM_List<T>* m_next;
   CSM_List<T>* m_prev;
};

//////////////////////////////////////////////////////////////////////////
// CSM_OID is the SFL OID class
class CSM_OID: public AsnOid
{
public:
   // CONSTRUCTORS
   CSM_OID() {};
   CSM_OID(char *pszOID) { PutChar(pszOID); }
   CSM_OID(const AsnOid &o) { Set (o); }

   void PutChar(char *szOidCopy); // set from a number-dot null term string
   char *GetChar();
   AsnOid *GetSNACCOid() { return this; }

   // OPERATORS
   operator char * () { return GetChar(); }
   operator const char * ()    { return (const char *)GetChar(); }
   bool operator == (const CSM_OID &o) {
         if (strcmp(GetChar(), (char *)(CSM_OID)o))
            return false;
         else
            return true; }
   bool operator != (const CSM_OID &o) {
         if (strcmp(GetChar(), (char *)(CSM_OID)o))
            return true;
         else
            return false; }
};

#endif // _SM_INTERNAL_H_
// EOF sm_internal.h
