/* @(#) sm_tlist.h 1.2 3/6/98 11:10:25 */

// CSM_List template source code.  Needs to be a header so that GCC on
// non-ELF systems will properly produce the object code.
//

template <class T>
CSM_List<T>::CSM_List(T *t)
{
   m_This = t;
   m_first = this;
   m_last = this;
   m_next = NULL;
   m_prev = NULL;
}

template <class T> void
CSM_List< T >::SelfDestruct(void)
{
   CSM_List<T> *temp = NULL;
   CSM_List<T> *new_first = NULL;
   CSM_List<T> *new_last = NULL;
   // if we delete this node then we must update
   // prev, next, and potentialy last or first
   delete m_This;

   // figure out new m_last
   if (m_last == this)
      // if this is the last node, then the new last node
      // will be the previous node, which will be NULL if
      // this is the only node
      new_last = m_prev;
   else
      new_last = m_last;

   if (m_first == this)
      m_first = NULL;

   // now, loop through entire list updating everybody
   temp = m_first;
   while (temp != NULL)
   {
      // if we're in the current node, don't do anything
      if (temp != this)
      {
         // update m_first
         if (new_first == NULL)
            new_first = temp;
         temp->m_first = new_first;

         // update m_last
         temp->m_last = new_last;

         // update m_next
         // if current node's next is this (the one being destroyed)
         if (temp->m_next == this)
            // then set the current node's next to this's next
            temp->m_next = m_next;

         // update m_prev
         // if current node's prev is this (the one being destroyed)
         if (temp->m_prev == this)
            // then set the current node's prev to this's prev
            temp->m_prev = m_prev;
      }
      temp = temp->m_next;
   }
   // now everybody's pointers are updated and this node
   // can be destroyed safely....
}

template <class T>
void CSM_List<T>::DestroyList()
{
   // find last node
   CSM_List<T> *temp = m_last;
   CSM_List<T> *del = NULL;

   // loop from end to start destroying as you go
   while (temp != NULL)
   {
      // save the current
      del = temp;
      // get the previous
      temp = temp->m_prev;
      // destroy the current
      delete del;
   }
}

template <class T>
void CSM_List<T>::Append(T *t)
{
   // temp pointer to last node in list
   CSM_List<T> *temp = m_last;
   // create a new node, store its address in last node's next
   // and put t into new node
   CSM_List<T> *new_node = temp->m_next = new CSM_List<T>(t);
   // store first node's address in new node
   new_node->m_first = m_first;
   // store last node's address in last node (itself)
   new_node->m_last = new_node;
   // store previous node's address (this) in new node;
   new_node->m_prev = temp;
   // update everybody's last pointer
   do {
      temp->m_last = new_node;
   } while ((temp = temp->m_prev) != NULL);
}

template <class T>
void CSM_List<T>::Append(CSM_List<T> *List)
{

   CSM_List<T> *node = List->First();

   do {
     Append(node->Get());
   } while ((node = node->Next()));
}

template <class T>
SM_RET_VAL CSM_List<T>::Length()
{
   SM_RET_VAL len = 0;
   CSM_List<T> *temp = m_first;

   while ((temp = temp->m_next) != NULL)
      ++len;
   return len;
}

template <class T>
CSM_List<T>* CSM_List<T>::First()
{
   return m_first;
}

template <class T>
CSM_List<T>* CSM_List<T>::Last()
{
   return m_last;
}

template <class T>
CSM_List<T>* CSM_List<T>::Next()
{
   return m_next;
}

template <class T>
CSM_List<T>* CSM_List<T>::Prev()
{
   return m_prev;
}

template <class T>
T* CSM_List<T>::Get()
{
   return m_This;
}

template <class T>
void CSM_List<T>::Set(T *t)
{
   m_This = t;
}

// EOF sm_list.cpp
