#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_Verify.cpp 1.14 3/26/98 12:28:28"; 
#endif

//
//  sm_Verify.cpp
//  These routines support the CSM_Sign class.

#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include "sm_api.h"


//
// CSM_SignMsg::Verify
//   This member function will verify the signature of the specified 
//   SignerInfos of an ASN.1 encoded SignedData message.
//
SM_RET_VAL CSM_VerifyMsg::Verify(CSMIME *pCsmime)
{
    SM_RET_VAL status=0;

    SME_SETUP("Sign");

    status = CSM_VerifyData::Verify(pCsmime, m_pEncodedBlob, m_pMsgCertCrls,
      NULL);
    
    SME_FINISH
    SME_CATCH_SETUP
        // local cleanup logic
    SME_CATCH_FINISH

    return(status);
}

//
//
SM_RET_VAL CSM_VerifyMsg::PreProc(CSMIME *pCSMIME)
{
    return CSM_VerifyData::PreProc(pCSMIME);
}


//
SM_RET_VAL CSM_VerifyMsg::PreProc(CSMIME *pCSMIME, CSM_Buffer *pCSMBlob)
{
   SM_RET_VAL status=0;
   SignerInfo *tmpSNACCSI;
   CSM_MsgSignerInfo *tmpSI;
   CSM_Buffer *pMessageDigest=NULL;

   SME_SETUP("CSM_VerifyMsg::PreProc");

   SME(status = CSM_VerifyData::PreProc(pCSMIME, pCSMBlob));
   if (status == 0 && m_pSnaccSignedData)
   {
      // LOAD encoded data
      if (m_pEncodedBlob)
         delete m_pEncodedBlob;
      SetEncodedBlob(pCSMBlob);   // assign to local encoded result.
      if (m_pMsgCertCrls)
         delete m_pMsgCertCrls;
      // LOAD certificates
      if (m_pSnaccSignedData->certificates)
         SME(m_pMsgCertCrls = new CSM_MsgCertCrls(
               m_pSnaccSignedData->certificates));
      // LOAD SignerInfos.
      for (m_pSnaccSignedData->signerInfos.SetCurrToFirst(),
           tmpSNACCSI=m_pSnaccSignedData->signerInfos.Curr(); 
           tmpSNACCSI; tmpSNACCSI=m_pSnaccSignedData->signerInfos.GoNext())
      {
         if ((tmpSI = new CSM_MsgSignerInfo(tmpSNACCSI)) == NULL)
            SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
         if (m_pSignerInfos == NULL)
         {
            if ((m_pSignerInfos = new CSM_MsgSignerInfos(tmpSI)) == NULL)
               SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
         }
         else
            m_pSignerInfos->Append(tmpSI);
         // RWC; TBD; Should now determine if any of the certificates in
         //   the SignedData message belong to this SignerInfo component
         //   and load into the "m_pSignerInfos" class.

         // Next, determine if there are any Authenticated Attributes,
         //  and check the "messageDigest" against our computed Hash.
         if (tmpSNACCSI->authenticatedAttributes != NULL)
         {
            if ((tmpSI->m_pAuthAttrs = new CSM_MsgAttributes) == NULL)
               SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
            tmpSI->m_pAuthAttrs->SetAuthEncodedAttrs(
                  tmpSNACCSI->authenticatedAttributes);
            tmpSI->m_pAuthAttrs->GetMessageDigest(pMessageDigest);
            if (pMessageDigest != NULL)
            {
               // check against computed hash.
            }
            else
            {
               // report error, missing messageDigest.
            }
         }
         // RWC; TBD; load Unauthenticated Attributes...
      }
   }

   SME_FINISH
   SME_CATCH_SETUP
      // local cleanup logic
   SME_CATCH_FINISH
   return status;
}

//
//
SM_RET_VAL CSM_VerifyData::PreProc(CSM_Buffer *pEncodedBlob) // IN, Resulting SignedData/ContentInfo.
{
    SME_SETUP("PreProc");
    if (pEncodedBlob)
    {
        if (m_pSnaccSignedData)
            delete m_pSnaccSignedData;
        m_pSnaccSignedData = new SignedData;
        SME(DECODE_BUF(m_pSnaccSignedData, pEncodedBlob));
    }
    SME_FINISH_CATCH
    return(0);
}

//
//
SM_RET_VAL CSM_VerifyData::PreProc(CSMIME *pCSMIME,
         CSM_Buffer *pEncodedBlob)     // IN, Resulting SignedData/ContentInfo.
{
    PreProc(pEncodedBlob);
    return(PreProc(pCSMIME));
}

//
//
SM_RET_VAL CSM_VerifyData::PreProc(CSMIME *pCSMIME)
{
    CSM_CSInstLst *tmpInst;
    SignedData *lpSignedData=m_pSnaccSignedData;
    SignerInfo *tmpSI;

    SME_SETUP("PreProc");
    {
        // RWC; check OIDs of SignedData against signature possibilities of
        //  CSMIME list of logon instances.
        pCSMIME->ClearFlag(SM_INST_APPLICABLE);
        for (tmpInst=pCSMIME->m_pCSInsts->First(); tmpInst; tmpInst=tmpInst->Next())
        {
            // check message OIDs with logon instance OIDs for matches.
            // (check each SignerInfo with each logon Instance).
            for (lpSignedData->signerInfos.SetCurrToFirst(),
                tmpSI=lpSignedData->signerInfos.Curr(); 
                tmpSI; tmpSI=lpSignedData->signerInfos.GoNext())
            {
                CSM_Alg alg1(*tmpSI->digestAlgorithm);
                CSM_Alg alg2(*tmpSI->signatureAlgorithm);
                if (tmpInst->Get()->FindAlgIds(&alg1, &alg2, NULL, NULL) )
                  tmpInst->Get()->SetApplicable();
                        // If the message Signature algs are found in this 
                        //   logon instance.
            }
        }
    }
    SME_FINISH_CATCH
    return(0);
}

//
//
SM_RET_VAL CSM_VerifyData::Verify(CSMIME *pCSMIME,   // IN,logged-on Instance list
      CSM_MsgCertCrls *pMsgCertCrls, // IN,Originator(s) certs+++
      CSM_Buffer *pSignedReceipt)   // OUT,returned receipt (if requested)
{
    SM_RET_VAL status=0;
    SignerInfo *tmpSI;

    if (m_pSnaccSignedData)
    {
        // Loop through all SignerInfos and call "Verify()".
        for (m_pSnaccSignedData->signerInfos.SetCurrToFirst(),
                tmpSI=m_pSnaccSignedData->signerInfos.Curr(); 
                tmpSI; tmpSI=m_pSnaccSignedData->signerInfos.GoNext())
        {
           status = Verify(pCSMIME, tmpSI, pMsgCertCrls, NULL, pSignedReceipt);
        }
    }

    return(status);
}

//
//
SM_RET_VAL CSM_VerifyData::Verify(CSMIME *pCSMIME,   // IN,logged-on Instance list
      CSM_Buffer *pEncodedBlob,      // IN, SignedData message
      CSM_MsgCertCrls *pMsgCertCrls, // IN,Originator(s) certs+++
      CSM_Buffer *pSignedReceipt)   // OUT,returned receipt (if requested)
{
    SM_RET_VAL status=0;

    if ((status=PreProc(pEncodedBlob)) == 0)
    {
        status = Verify(pCSMIME, pMsgCertCrls, pSignedReceipt);
    }

    return(status);
}

//
//
CSM_CSInst *CSM_VerifyData::GetSignerInst(
    CSMIME *pCSMIME,  // IN, list of logons
    SignerInfo *pSI)  // IN, specific SignerInfo
{
    CSM_CSInstLst *tmpInst;
    CSM_CSInst *pResultInst=NULL;

    SME_SETUP("PreProc");

    for (tmpInst=pCSMIME->m_pCSInsts->First(); tmpInst && pResultInst == NULL; tmpInst=tmpInst->Next())
    {
                CSM_Alg alg1(*pSI->digestAlgorithm);
                CSM_Alg alg2(*pSI->signatureAlgorithm);
                printf("GetSignerInst: digest=%s signature=%s\n", 
                    alg1.GetId()->GetChar(), alg2.GetId()->GetChar());
                if (tmpInst->Get()->IsApplicable() && 
                    tmpInst->Get()->FindAlgIds(&alg1, &alg2, NULL, NULL) )
                    pResultInst = tmpInst->Get();
                        // If the message Signature algs are found in this 
                        //   logon instance.
                  
    }
    SME_FINISH_CATCH
    return(pResultInst);
}

//
// ExtractSignerPublicKey
// This routine will extract the signer public key from the specified certificate.
//  The certificate is expected to be found in the application specified
//  pMsgCertCrls or the SignedData message itself.  If the public key is not 
//  found, then a NULL is returned for the buffer.
CSM_Buffer *CSM_VerifyData::ExtractSignerPublicKey(
    IssuerAndSerialNumber *issuerAndSerialNumber, // IN, certificate ID
    CSM_MsgCertCrls *pMsgCertCrls)     // IN,Originator(s) certs+++
{
    CSM_Buffer *pResultBuf=NULL;
    CSM_CertificateChoiceLst *tmpCertList;
    CSM_CertificateChoiceLst *pCerts;
    CSM_CertificateChoice *pCert;
    Certificate SNACCCertificate;
    Certificate *pResultCert=NULL;
    CertificateChoices *tmpSNACCCert;

    SME_SETUP("ExtractSignerPublicKey");

    if (issuerAndSerialNumber == NULL)
    {
         SME_THROW(22, "issuerAndSerialNumber missing parameter", NULL);
    }
    // First, search for the certificate in the user specified buffer
    if (pMsgCertCrls)
    {
        if ((pCerts=pMsgCertCrls->GetCertificates()) != NULL)
        {
              for (tmpCertList=pCerts->First(); 
                tmpCertList && pResultCert == NULL;
                tmpCertList=tmpCertList->Next())
              {               // For each cert buffer, check for specified 
                              //  issuer and serial number.
                    pCert = tmpCertList->Get();
                    if (pCert->m_lCertType == CSM_CertificateChoice::CERT_TYPE_CERT)
                    {               // ONLY check Certificates, not ACs or Extended.
                      DECODE_BUF(&SNACCCertificate, 
                           (*(pCert->GetEncodedCert())));

// G++ doesn't like this -PL
//                      if (CSM_IssuerAndSerialNumber(SNACCCertificate) == 
//                          CSM_IssuerAndSerialNumber(*issuerAndSerialNumber))
//                      {           // found proper certificate.
//                          pResultCert = &SNACCCertificate;
//                      }

                      CSM_IssuerAndSerialNumber 
                           tmpIASN1(SNACCCertificate);
                      CSM_IssuerAndSerialNumber 
                           tmpIASN2(*issuerAndSerialNumber);

                      if (tmpIASN1 == tmpIASN2)
                      {           // found proper certificate.
                         pResultCert = &SNACCCertificate;
                      }
                   }
              }
        }

    }

    if (pResultCert == NULL)   // if not in parameter, then check the message.
    {
        if (m_pSnaccSignedData && m_pSnaccSignedData->certificates)
        {
            for (m_pSnaccSignedData->certificates->SetCurrToFirst(),
                tmpSNACCCert=m_pSnaccSignedData->certificates->Curr();
                tmpSNACCCert;  tmpSNACCCert=m_pSnaccSignedData->certificates->GoNext())
            {
                if (tmpSNACCCert->choiceId == CertificateChoices::certificateCid)
                {

// G++ doesn't like this -PL
//                    if (CSM_IssuerAndSerialNumber(*tmpSNACCCert->certificate)
//                        == CSM_IssuerAndSerialNumber(*issuerAndSerialNumber))
//                    {           // found proper certificate.
//                          pResultCert = tmpSNACCCert->certificate;
//                    }
                   CSM_IssuerAndSerialNumber 
                         tmpIASN1(*tmpSNACCCert->certificate);
                   CSM_IssuerAndSerialNumber 
                         tmpIASN2(*issuerAndSerialNumber);

                   if (tmpIASN1 == tmpIASN2)
                   {    // found proper certificate.
                      pResultCert = tmpSNACCCert->certificate;
                   }
                }
            }
        }
    }

    if (pResultCert)
    {
        // RWC; TBD; validate that 
        //  SNACCCertificate.certificateToSign->algorithm->
        //  is valid, matches SignerInfo??.
        if (pResultCert->certificateToSign && 
            pResultCert->certificateToSign->subjectPublicKeyInfo)
        {
          pResultBuf = new CSM_Buffer;
          SME(SM_AsnBits2Buffer(
           &pResultCert->certificateToSign->subjectPublicKeyInfo->subjectPublicKey, 
           pResultBuf));
          SME(SM_BufferReverseBits(pResultBuf)); 
            // re-reverse bits to keep aligned for Public key.
                          //  SNACCCertificate.certificateToSign->
                          //  subjectPublicKeyInfo->subjectPublicKey;
                          // AsnBits...
        }
    }

    SME_FINISH
    SME_CATCH_SETUP
        // local cleanup logic
        // NO FREE OF "pResultCert" is necessary; in 1 case destroyed on 
        //     method return, in 2nd case pointer to SignedData element.
    SME_CATCH_FINISH

    return(pResultBuf);
}

//
//  This method verifies the specified logon instance, if a matching SignerInfo
//  is located.
SM_RET_VAL CSM_VerifyData::Verify(
    CSMIME *pCSMIME,               // IN, list of logons
    SignerInfo *pSI,               // IN, specific SignerInfo to process.
    CSM_MsgCertCrls *pMsgCertCrls, // IN,Originator(s) certs+++
    CSM_MsgAttributes *pAuthAttrs, // IN, optional signed attributes
    CSM_Buffer *pSignedReceipt)    // OUT,returned receipt (if requested)
{
    SM_RET_VAL status=2;        // default return INSTANCE not found.
    CSM_CSInst *pInst=NULL;
    CSM_Buffer *tmpContentBuf=NULL;
    CSM_Buffer *pSignerPublicKey;

    SME_SETUP("Verify");
    // match up the SignerInfo to a logon instance.
    if ((pInst = GetSignerInst(pCSMIME, pSI)) != NULL)
    {
        // Now, we have located a valid logon instance to verify the signature.
        if (m_pSnaccSignedData->encapContentInfo)  // if there is a content to verify.
        {
            CSM_Alg alg1;//RWC;(pSI->digestAlgorithm);
            CSM_Alg alg2;//RWC;(pSI->signatureAlgorithm);
            // prepare for verification; load CTI style parameters from 
            //  SNACC classes.
            tmpContentBuf = new CSM_Buffer;
            CSM_Buffer tmpSig((char *)pSI->signature, pSI->signature.Len());

            SME(ENCODE_BUF(m_pSnaccSignedData->encapContentInfo,
                tmpContentBuf));
            // Extract the Signer's public key from the certificate (either 
            //  provided in the paraemter "pMsgCertCrls" or contained in the 
            //  message.  (See CSM_VerifyData::ExtractSignerPublicKey()).
            SME(pSignerPublicKey = 
                ExtractSignerPublicKey(pSI->issuerAndSerialNumber,
                pMsgCertCrls));
            // RWC; TBD; If authenticated attributes are present,
            //   Check a newly computed HASH value against the "messageDigest"
            //   value provided by the originator.  This is not important for 
            //   the "SMTI_Verify()" call, but is necessary for the SMIME ops.
            if (pAuthAttrs)   // As loaded by the PreProc() call.
            {                 //  Only checked if passed in by application.

            }
            // Verify the signature of this particular SignerInfo.
            if (pSignerPublicKey != NULL)
            {
              SME(status = pInst->GetTokenInterface()->
                SMTI_Verify(pSignerPublicKey, &alg1, &alg2, tmpContentBuf, 
                &tmpSig));
            }
            else
            {
              delete tmpContentBuf;
              SME_THROW(22, "MUST HAVE SignerPublicKey from certificate",NULL);
            }
            delete tmpContentBuf;
            // RWC; TBD; Add logic to flag successfully verified SignerInfos
        }
        else
           SME_THROW(22, "SNACC SignedData missing content", NULL);
    }
    else
        printf("CSM_VerifyData::Verify: no session instance found for SignerInfo.\n");

    SME_FINISH_CATCH

    return(status);

}



// EOF sm_Verify.cpp
