#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_Alg.cpp 1.7 3/25/98 10:28:21"; 
#endif

//////////////////////////////////////////////////////////////////////////
// sm_Alg.cpp
//////////////////////////////////////////////////////////////////////////

#include "sm_api.h"

//////////////////////////////////////////////////////////////////////////
void CSM_Alg::Clear()
{
   m_pId = NULL; 
   m_pParams = NULL; 
   m_pSNACCAlgId = NULL;
}

//////////////////////////////////////////////////////////////////////////
CSM_Alg::CSM_Alg()
{
   Clear();
}

//////////////////////////////////////////////////////////////////////////
CSM_Alg::CSM_Alg(CSM_OID &AlgOid) 
{ 
   SME_SETUP("CSM_Alg::CSM_Alg(CSM_OID &)");

   Clear();
   if ((m_pId = new CSM_OID(AlgOid)) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

   SME_FINISH_CATCH;
}

//////////////////////////////////////////////////////////////////////////
CSM_Alg::CSM_Alg(AlgorithmIdentifier &SNACCAlgId) 
{
   SME_SETUP("CSM_Alg::CSM_Alg(AlgorithmIdentifier &)");

   Clear();
   
   if ((m_pSNACCAlgId = new AlgorithmIdentifier) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
   *m_pSNACCAlgId = SNACCAlgId;
   if ((m_pId = new CSM_OID(m_pSNACCAlgId->algorithm)) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
   if (m_pSNACCAlgId->parameters)
   {
      SM_EXTRACT_ANYBUF(m_pParams, m_pSNACCAlgId->parameters); 
   }
   else
      m_pParams = NULL;

   SME_FINISH_CATCH;
}

//////////////////////////////////////////////////////////////////////////
CSM_OID* CSM_Alg::GetId()
{ 
   return m_pId; 
}

//////////////////////////////////////////////////////////////////////////
AlgorithmIdentifier* CSM_Alg::GetSNACCAlgId() 
{
   SME_SETUP("CSM_Alg::GetSNACCAlgId");

   if (m_pSNACCAlgId == NULL)
   { 
      if ((m_pSNACCAlgId = new AlgorithmIdentifier) == NULL)
         SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

      m_pSNACCAlgId->algorithm = *m_pId->GetSNACCOid();
      if (m_pParams != NULL)
      {
         m_pSNACCAlgId->parameters = new AsnAny;
         SM_ASSIGN_ANYBUF(m_pParams, m_pSNACCAlgId->parameters);
      }
   }

   SME_FINISH_CATCH;

   return m_pSNACCAlgId; 
}

//////////////////////////////////////////////////////////////////////////
bool CSM_Alg::operator == (CSM_Alg &AlgId) 
{
   bool result = true;

   //GetId();
   //GetParams();

   if (m_pId && AlgId.m_pId)
   {  
      if (*m_pId != *AlgId.m_pId)
         result = false;
   }
   else if (m_pId != NULL || AlgId.m_pId != NULL)
      result = false;  // one is NULL, other is not, !=.
   if (result && m_pParams && AlgId.m_pParams)
   {
      if (*m_pParams != *AlgId.m_pParams)
         result = false;
   }
   else if (result && 
         (m_pParams != NULL || AlgId.m_pParams != NULL))
      result = false;  // one is NULL, other is not, !=.
   return(result); 
}

//////////////////////////////////////////////////////////////////////////
bool CSM_Alg::operator != (CSM_Alg &AlgId) 
{ 
   return !(*this == AlgId); 
}

//////////////////////////////////////////////////////////////////////////
void CSM_Alg::PutSNACCAlg(AlgorithmIdentifier *pSNACC_Alg)
{
   SME_SETUP("CSM_Alg::PutSNACCAlg");

   if (pSNACC_Alg == NULL)
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);
   // m_pId is a required parameter.  If it's not set error out
   if (m_pId == NULL)
      SME_THROW(SM_MISSING_PARAM, "Alg doesn't have an OID", NULL);
   pSNACC_Alg->algorithm.Set(*(m_pId->GetSNACCOid()));

   // parameter are optional so coniditionally add it to the snacc
   // class.
   // Load Params from CSM_Buffer
   if (m_pParams != NULL)
      SM_ASSIGN_ANYBUF(m_pParams, pSNACC_Alg->parameters);
   SME_FINISH_CATCH
}

// EOF sm_Alg.cpp
