#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_BaseTokenInterface.cpp 1.8 3/25/98 17:31:48"; 
#endif

// sm_BaseTokenInterface.cpp
#include "sm_api.h"

//////////////////////////////////////////////////////////////////////////
void CSM_BaseTokenInterface::BTISetPreferredCSInstAlgs(CSM_OID *oidDigest,
            CSM_OID *oidDigestEncryption,
            CSM_OID *oidKeyEncryption,
            CSM_OID *oidContentEncryption)
{
   if (oidDigest != NULL)
      m_digestOID = *oidDigest;
   if (oidDigestEncryption != NULL)
      m_digestEncryptionOID = *oidDigestEncryption;
   if (oidKeyEncryption != NULL)
      m_keyEncryptionOID = *oidKeyEncryption;
   if (oidContentEncryption != NULL)
      m_contentEncryptionOID = *oidContentEncryption;
}

//////////////////////////////////////////////////////////////////////////
// This function returns the preferred OIDs from the base token interface.
// if any of the incoming pointers to references are Null, that one is
// not set....
void CSM_BaseTokenInterface::BTIGetPreferredCSInstAlgs(CSM_OID *&oidDigest,
            CSM_OID *&oidDigestEncryption,
            CSM_OID *&oidKeyEncryption,
            CSM_OID *&oidContentEncryption)
{
   if (oidDigest != NULL)
      *oidDigest = m_digestOID;
   if (oidDigestEncryption != NULL)
      *oidDigestEncryption = m_digestEncryptionOID;
   if (oidKeyEncryption != NULL)
      *oidKeyEncryption = m_keyEncryptionOID;
   if (oidContentEncryption != NULL)
      *oidContentEncryption = m_contentEncryptionOID;
}

//////////////////////////////////////////////////////////////////////////
// if any parameter to this function is NULL then it ignores that part of
// processing.  This function allows the caller to set the alg ID lists
// in the base token interface.  If an alg id list is already in the
// base token interface, then this function free's it before setting
// the new one
void CSM_BaseTokenInterface::BTISetAlgIDs(CSM_AlgLst *pdigestAlgID, 
            CSM_AlgLst *pdigestEncryptionAlgID,
            CSM_AlgLst *pkeyEncryptionAlgID,
            CSM_AlgLst *pcontentEncryptionAlgID)
{
   if (pdigestAlgID != NULL)
   {
      if (m_pDigestAlgorithms != NULL && 
          m_pDigestAlgorithms != pdigestAlgID)
         delete m_pDigestAlgorithms;
      m_pDigestAlgorithms = pdigestAlgID;
   }
   if (pdigestEncryptionAlgID != NULL)
   {
      if (m_pDigestEncryptionAlgorithms != NULL && 
          m_pDigestEncryptionAlgorithms != pdigestEncryptionAlgID)
         delete m_pDigestEncryptionAlgorithms;
      m_pDigestEncryptionAlgorithms = pdigestEncryptionAlgID;
   }
   if (pkeyEncryptionAlgID != NULL)
   {
      if (m_pKeyEncryptionAlgorithms != NULL &&
          m_pKeyEncryptionAlgorithms != pkeyEncryptionAlgID)
         delete m_pKeyEncryptionAlgorithms;
      m_pKeyEncryptionAlgorithms = pkeyEncryptionAlgID;
   }
   if (pcontentEncryptionAlgID != NULL)
   {
      if (m_pContentEncryptionAlgorithms != NULL &&
          m_pContentEncryptionAlgorithms != pcontentEncryptionAlgID)
         delete m_pContentEncryptionAlgorithms;
      m_pContentEncryptionAlgorithms = pcontentEncryptionAlgID;
   }
}

//////////////////////////////////////////////////////////////////////////
// if any parameter to this function is NULL then this function ignores
// that part of the processing.  This function allows the caller to
// retrieve the alg ID lists from the base token interface.
void CSM_BaseTokenInterface::BTIGetAlgIDs(CSM_AlgLst **ppdigestAlgID, 
            CSM_AlgLst **ppdigestEncryptionAlgID,
            CSM_AlgLst **ppkeyEncryptionAlgID,
            CSM_AlgLst **ppcontentEncryptionAlgID)
{
   if (ppdigestAlgID != NULL)
      *ppdigestAlgID = m_pDigestAlgorithms;
   if (ppdigestEncryptionAlgID != NULL)
      *ppdigestEncryptionAlgID = m_pDigestEncryptionAlgorithms;
   if (ppkeyEncryptionAlgID != NULL)
      *ppkeyEncryptionAlgID = m_pKeyEncryptionAlgorithms;
   if (ppcontentEncryptionAlgID != NULL)
      *ppcontentEncryptionAlgID = m_pContentEncryptionAlgorithms;
}

//////////////////////////////////////////////////////////////////////////
// BTIFindAlgIds will return true if all of the specified algorithm ids 
// are in this base token interface.  Parameters that are null are 
// ignored.
// TBD, maybe we should just pass OIDs in instead of AlgIDs???
bool CSM_BaseTokenInterface::BTIFindAlgIds(CSM_Alg *pdigestAlgID, 
            CSM_Alg *pdigestEncryptionAlgID,
            CSM_Alg *pkeyEncryptionAlgID,
            CSM_Alg *pcontentEncryptionAlgID)
{
   CSM_AlgLst *pTemp;
   bool bRet = true;
   bool bFoundThisOne;

   while (true)
   {
      // find out if the requested digest alg is in the list
      if (pdigestAlgID != NULL)
      {
         if (m_pDigestAlgorithms != NULL)
         {
            // search for matching digest alg
            pTemp = m_pDigestAlgorithms->First();
            bFoundThisOne = false;
            do
            {
               if (*pTemp->Get() == *pdigestAlgID)
               {
                  bFoundThisOne = true;
                  break;
               }
            } while ((pTemp = pTemp->Next()) != NULL);
            if (!bFoundThisOne)
            {
               // requested oid not found in list
               bRet = false;
               break;
            }
         }
         else
         {
            // requested oid not found in list because list doesn't exist
            bRet = false;
            break;
         }
      }

      // find out if the requested digest encryption alg is in the list
      if (pdigestEncryptionAlgID != NULL)
      {
         if (m_pDigestEncryptionAlgorithms != NULL)
         {
            // search for matching digest encryption alg
            pTemp = m_pDigestEncryptionAlgorithms->First();
            bFoundThisOne = false;
            do
            {
               if (*pTemp->Get() == *pdigestEncryptionAlgID)
               {
                  bFoundThisOne = true;
                  break;
               }
            } while ((pTemp = pTemp->Next()) != NULL);
            if (!bFoundThisOne)
            {
               // requested oid not found in list
               bRet = false;
               break;
            }
         }
         else
         {
            // requested oid not found in list because list doesn't exist
            bRet = false;
            break;
         }
      }

      // find out if the requested key encryption alg is in the list
      if (pkeyEncryptionAlgID != NULL)
      {
         if (m_pKeyEncryptionAlgorithms != NULL)
         {
            // search for matching key encryption alg
            pTemp = m_pKeyEncryptionAlgorithms->First();
            bFoundThisOne = false;
            do
            {
               if (*pTemp->Get() == *pkeyEncryptionAlgID)
               {
                  bFoundThisOne = true;
                  break;
               }
            } while ((pTemp = pTemp->Next()) != NULL);
            if (!bFoundThisOne)
            {
               // requested oid not found in list
               bRet = false;
               break;
            }
         }
         else
         {
            // requested oid not found in list because list doesn't exist
            bRet = false;
            break;
         }
      }

      // find out if the requested content encryption alg is in the list
      if (pcontentEncryptionAlgID != NULL)
      {
         if (m_pContentEncryptionAlgorithms != NULL)
         {
            // search for matching content encryption alg
            pTemp = m_pContentEncryptionAlgorithms->First();
            bFoundThisOne = false;
            do
            {
               if (*pTemp->Get() == *pcontentEncryptionAlgID)
               {
                  bFoundThisOne = true;
                  break;
               }
            } while ((pTemp = pTemp->Next()) != NULL);
            if (!bFoundThisOne)
            {
               // requested oid not found in list
               bRet = false;
               break;
            }
         }
         else
         {
            // requested oid not found in list because list doesn't exist
            bRet = false;
            break;
         }
      }

      break; // break out under normal condition
   }

   return bRet;
}

//////////////////////////////////////////////////////////////////////////
void CSM_BaseTokenInterface::ClearDigestEncryptionCapability()
{
   CSM_OID oidEmpty;

   m_digestEncryptionOID = oidEmpty;
   if (m_pDigestEncryptionAlgorithms)
   {
      delete(m_pDigestEncryptionAlgorithms);
      m_pDigestEncryptionAlgorithms = NULL;
   }
}

void CSM_BaseTokenInterface::ClearKeyEncryptionCapability()
{
   memset(&m_keyEncryptionOID, 0, sizeof(CSM_OID));
   if (m_pKeyEncryptionAlgorithms)
   {
      delete(m_pKeyEncryptionAlgorithms);
      m_pKeyEncryptionAlgorithms = NULL;
   }
}

// EOF sm_BaseTokenInterface.cpp
