#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_CSInst.cpp 1.10 3/6/98 18:30:35"; 
#endif

//
//  sm_CSInst.cpp
//  This set of C++ routines support the CSM_CSInst class.
//

#include "sm_api.h"
//CSM_List<CSM_Buffer>;


//
CSM_CSInst::CSM_CSInst()
{
    m_UseThis = false;
    m_Applicable = false;
    m_pCertificates = NULL;
    m_pCRLs = NULL;
    m_pErrorBuf = NULL;
    m_pTokenInterface = NULL;
    m_pszCSInstID = NULL;
    m_pIssuerAndSerialNumber = NULL;
    m_pSubjectDN = NULL;
}

//
CSM_CSInst::~CSM_CSInst()
{
    if (m_pTokenInterface)
    {
       // alg lists destroyed through destruction of m_pTokenInterface
       // which is derived from CSM_BaseTokenInterface
       // delete CSM_TokenInterface *m_pTokenInterface;
    }
    if (m_pszCSInstID)
       free(m_pszCSInstID);
    if (m_pCertificates)
    {
        // free Cert_path_LL *m_Certificates;
    }
    if (m_pCRLs)
    {
        // free SM_Crl_lst *m_CRLs;
    }
    if (m_pIssuerAndSerialNumber)
    {
        delete m_pIssuerAndSerialNumber;
    }
}


//
void CSM_CSInst::SetTokenInterface(CSM_TokenInterface *pTokenInterface)
{
    m_pTokenInterface = pTokenInterface;
}


//
void CSM_CSInst::SetID(char *pszSessionID)
{
    m_pszCSInstID = strdup(pszSessionID);
}



//  SetCertificates
//  This method assumes all specified Certs are certificates, not
//  Attribute Certs nor Extended Certs.
void CSM_CSInst::SetCertificates(CSM_BufferLst  *pCertificates)
{
    CSM_BufferLst *ttt;
    CSM_Buffer *tmpBufFile;
    CSM_CertificateChoice *tmpCert;

    for (ttt=pCertificates->First()/*->Get()*/; ttt;
         ttt=pCertificates->Next()/*->Get()*/)
    {
        tmpBufFile = ttt->Get();
        tmpCert = new CSM_CertificateChoice(tmpBufFile);
        if (!m_pCertificates)
          m_pCertificates = new CSM_CertificateChoiceLst(tmpCert);
        else
          m_pCertificates->Append(tmpCert);
    }
}

void CSM_CSInst::SetCertificates(CSM_CertificateChoiceLst *pCertificates)
{
   m_pCertificates = pCertificates;
}


//
void CSM_CSInst::SetCRLs(CSM_BufferLst *pCRLs)
{
    m_pCRLs = pCRLs;
}


//
void CSM_CSInst::SetIssuerAndSerialNumber(CSM_IssuerAndSerialNumber 
            *pIssuerAndSerialNumber)
{
    if (m_pIssuerAndSerialNumber)
        delete m_pIssuerAndSerialNumber ;
    m_pIssuerAndSerialNumber = new 
        CSM_IssuerAndSerialNumber(*pIssuerAndSerialNumber);
}


CSM_IssuerAndSerialNumber *CSM_CSInst::GetIssuerAndSerialNumber()
{
    CSM_CertificateChoiceLst *tmpCsmbCertLst=NULL;
    
    SME_SETUP("sm_PutSignerInfo");
    if (m_pIssuerAndSerialNumber)           // remove old if present.
        delete m_pIssuerAndSerialNumber;
    m_pIssuerAndSerialNumber = NULL;
    if (m_pCertificates)
    {
      for (tmpCsmbCertLst=m_pCertificates->First(); tmpCsmbCertLst &&
          tmpCsmbCertLst->Get()->m_lCertType != 
          CSM_CertificateChoice::CERT_TYPE_CERT; 
        tmpCsmbCertLst=tmpCsmbCertLst->Next());
      if (tmpCsmbCertLst)       // If there is a User certificate.
      {
          m_pIssuerAndSerialNumber = 
             new CSM_IssuerAndSerialNumber(tmpCsmbCertLst->Get()->
                  GetEncodedCert());
      }
    }

    SME_FINISH_CATCH

    return(m_pIssuerAndSerialNumber);
}


//

// SetPreferrredSessionAlgs will set the preferred encryption and signing
//  algorithms by OID.  They must be part of the listed algorithms supported
//  by this session or the member function will return an error (non-zero).
//  NULL parameter are acceptable to allow the default session algorithms.
void CSM_CSInst::SetPreferredCSInstAlgs(CSM_OID *oidDigest,
            CSM_OID *oidDigestEncryption,
            CSM_OID *oidKeyEncryption,
            CSM_OID *oidContentEncryption)
{
   // RWC; TBD; check that these OIDs reside in alglists.
   m_pTokenInterface->BTISetPreferredCSInstAlgs(oidDigest,
         oidDigestEncryption, oidKeyEncryption, oidContentEncryption);
}

void CSM_CSInst::GetPreferredCSInstAlgs(CSM_OID *oidDigest,
                                        CSM_OID *oidDigestEncryption,
                                        CSM_OID *oidKeyEncryption,
                                        CSM_OID *oidContentEncryption)
{
   m_pTokenInterface->BTIGetPreferredCSInstAlgs(oidDigest,
         oidDigestEncryption, oidKeyEncryption, oidContentEncryption);
}

//
void CSM_CSInst::SetAlgIDs(CSM_AlgLst *pdigestAlgID, 
            CSM_AlgLst *pdigestEncryptionAlgID,
            CSM_AlgLst *pkeyEncryptionAlgID,
            CSM_AlgLst *pcontentEncryptionAlgID)
{
   m_pTokenInterface->BTISetAlgIDs(pdigestAlgID,
         pdigestEncryptionAlgID,
         pkeyEncryptionAlgID,
         pcontentEncryptionAlgID);
}

//
void CSM_CSInst::GetAlgIDs(CSM_AlgLst *&pdigestAlgID, 
            CSM_AlgLst *&pdigestEncryptionAlgID,
            CSM_AlgLst *&pkeyEncryptionAlgID,
            CSM_AlgLst *&pcontentEncryptionAlgID)
{
    CSM_AlgLst **ppdigestAlgID=NULL,
         **ppdigestEncryptionAlgID=NULL,
         **ppkeyEncryptionAlgID=NULL,
         **ppcontentEncryptionAlgID=NULL;
    if (pdigestAlgID) 
        ppdigestAlgID = &pdigestAlgID;
    if (pdigestEncryptionAlgID) 
        ppdigestEncryptionAlgID = &pdigestEncryptionAlgID;
    if (pkeyEncryptionAlgID) 
        ppkeyEncryptionAlgID = &pkeyEncryptionAlgID;
    if (pcontentEncryptionAlgID)
        ppcontentEncryptionAlgID = &pcontentEncryptionAlgID;
   m_pTokenInterface->BTIGetAlgIDs(ppdigestAlgID,
         ppdigestEncryptionAlgID,
         ppkeyEncryptionAlgID,
         ppcontentEncryptionAlgID);
}

//
// FindAlgIds
// This function searches for any specified algorithm IDs
// in the appropriate private members algorithm lists for the specified
// algorithm.  If all specified algorithms are found, a TRUE is returned.
// Any of the parameters can be NULL; it is expected that at least one is 
// not NULL.
bool CSM_CSInst::FindAlgIds(CSM_Alg *pdigestAlgID, 
            CSM_Alg *pdigestEncryptionAlgID,
            CSM_Alg *pkeyEncryptionAlgID,
            CSM_Alg *pcontentEncryptionAlgID)
{
   return (m_pTokenInterface->BTIFindAlgIds(pdigestAlgID, pdigestEncryptionAlgID,
         pkeyEncryptionAlgID, pcontentEncryptionAlgID));
}

//
//
CSM_DN *CSM_CSInst::GetSubjectDN()
{   
   Certificate tmpSNACCCert;
   CSM_CertificateChoiceLst *tmpCsmbCertLst=NULL;

   if (m_pCertificates != NULL)
   {
      for (tmpCsmbCertLst=m_pCertificates->First(); tmpCsmbCertLst &&
          tmpCsmbCertLst->Get()->m_lCertType != 
          CSM_CertificateChoice::CERT_TYPE_CERT; 
        tmpCsmbCertLst=tmpCsmbCertLst->Next());
      if (tmpCsmbCertLst)       // If there is a User certificate.
      {
          if (m_pSubjectDN)
               delete m_pSubjectDN;
          m_pSubjectDN = NULL;
          DECODE_BUF(&tmpSNACCCert, 
               (*(tmpCsmbCertLst->Get()->GetEncodedCert())));
          if (tmpSNACCCert.certificateToSign->subject)
           m_pSubjectDN = new CSM_DN(*tmpSNACCCert.certificateToSign->subject);
      }
   }

   return m_pSubjectDN; 
}



#ifdef NOT_USED_NOW
//
SM_RET_VAL CSM_CSInst::Login(CSMIME *pCSMIME)
{
    SM_RET_VAL status=0;

    mp_ErrorBuf = pCSMIME->GetErrorBuf();  // point to library class error buffer;
                                       //  all sessions will report to this
                                       //  buffer for application level 
                                       //  processing.
    if (m_pTokenInterface)
    {
        status = m_pTokenInterface->SMTI_Login();
    }
    else
    {
        // report error, not token interface.
    }

    return(status);
}
#endif

// sm_CSInst.cpp
