#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_ContentInfo.cpp 1.2 3/18/98 10:00:50"; 
#endif

//
// sm_ContentInfo.cpp
// This file provides the ContentInfo class support functionality.
//
//

#include "sm_api.h"



CSM_ContentInfoMsg::CSM_ContentInfoMsg()
{
    m_pSNACCContentInfo = NULL;
}

// This constructor builds a CSM_ContentInfo class from a SNACC ContentInfo
//  ASN.1 encoded Buffer (NOT CONTENT).
CSM_ContentInfoMsg::CSM_ContentInfoMsg(CSM_Buffer *pMessageBuf)
{
    CSM_Buffer *pContentBuf;
    AsnOcts SNACCOcts;

    SME_SETUP("CSM_ContentInfoMsg: CSM_Buffer");
    SME(SetEncodedBlob(pMessageBuf));
    m_pSNACCContentInfo = new ContentInfo;
    SME(DECODE_BUF(m_pSNACCContentInfo, pMessageBuf));
    SME(SM_EXTRACT_ANYBUF(pContentBuf, m_pSNACCContentInfo->content));
    if (m_pSNACCContentInfo->contentType == id_data)    
    {               // perform decode of OCTET STRING
        DECODE_BUF(&SNACCOcts, pContentBuf);
        delete pContentBuf;
        pContentBuf = new CSM_Buffer((char *)SNACCOcts, SNACCOcts.Len());
                    // Only assigne Octet contents, already decoded for user.
    }
    SME(m_pContentValue = new CSM_Content(pContentBuf, 
        CSM_OID(m_pSNACCContentInfo->contentType)));
    delete pContentBuf;
    SME_FINISH_CATCH
}




bool CSM_ContentInfoMsg::IsSignedData()
{
   bool result=false;

    if (m_pContentValue->m_contentType == id_signedData)
        result = true;

   return(result);
}


bool CSM_ContentInfoMsg::IsEnvelopedData()
{
   bool result=false;

    if (m_pContentValue->m_contentType == id_envelopedData)
        result = true;

   return(result);
}


bool CSM_ContentInfoMsg::IsData()
{
   bool result=false;

    if (m_pContentValue->m_contentType == id_data)
        result = true;
   return(result);
}


// This method will encode the ASN.1 ContentInfo from the member variables
//  as necessary.  It is expected that any data changes will clear the 
//  "m_pEncodedBlob" member variable.
CSM_Buffer *CSM_ContentInfoMsg::GetEncodedCI()
{
    AsnOcts SNACCOcts;
    CSM_Buffer *pTmpContentBuf=NULL;

    SME_SETUP("GetEncodedCI");
    if (m_pEncodedBlob == NULL && m_pContentValue != NULL)
    {                       // then attempt to encode the contents.
        if (m_pSNACCContentInfo != NULL)
            delete m_pSNACCContentInfo;
        m_pSNACCContentInfo = new ContentInfo;
        m_pSNACCContentInfo->contentType = m_pContentValue->m_contentType;
        if (m_pSNACCContentInfo->contentType == id_data)    
        {               // perform encode of OCTET STRING
            SME(SNACCOcts.Set((char *)m_pContentValue->m_content, 
                m_pContentValue->m_content.Length()));
            SME(ENCODE_BUF(&SNACCOcts, pTmpContentBuf));
        }
        else        // just send full existing ANY buffer.
        {
            pTmpContentBuf = &m_pContentValue->m_content;
        }
        m_pSNACCContentInfo->content = new AsnAny;//DefinedBy;
        SME(SM_ASSIGN_ANYBUF(pTmpContentBuf, m_pSNACCContentInfo->content));
        SME(ENCODE_BUF(m_pSNACCContentInfo, m_pEncodedBlob));
        if (pTmpContentBuf != &m_pContentValue->m_content)
            delete pTmpContentBuf;  // ONLY delete if id_data type.
    }
    SME_FINISH_CATCH
    return(m_pEncodedBlob);
}

void CSM_ContentInfoMsg::SetEncodedCI(CSM_Buffer &Buffer)
{
    CSM_Buffer *pTmpBuf=NULL;

    SME_SETUP("SetEncodedCI");
    // FIRST, assign buffer of ASN.1 encoded ContentInfo
    if (m_pEncodedBlob)
        delete m_pEncodedBlob;
    m_pEncodedBlob = new CSM_Buffer(Buffer);
    // NEXT, decode ContentInfo
    if (m_pSNACCContentInfo)
        delete m_pSNACCContentInfo;
    m_pSNACCContentInfo = new ContentInfo;
    DECODE_BUF(m_pSNACCContentInfo, m_pEncodedBlob);
    // NOW, load up the content details.
    SME(SM_EXTRACT_ANYBUF(pTmpBuf, m_pSNACCContentInfo->content));
    if (m_pContentValue != NULL)
        delete m_pContentValue;
    m_pContentValue =new CSM_Content(pTmpBuf,
        CSM_OID(m_pSNACCContentInfo->contentType));
    if (pTmpBuf)
        delete pTmpBuf;

    SME_FINISH_CATCH
}

// EOF sm_ContentInfo.cpp
