#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_CsmSupport.cpp 1.13 3/25/98 10:27:59"; 
#endif
#include "sm_api.h"

// sm_CsmSupport.cpp
// Various support functions

// The following CSM_IssuerAndSerialNumber member functions are defined here:
// Constructors
// bool operator ==

// use this constructor to make a copy of the provided structure
// and put it into this structure
CSM_IssuerAndSerialNumber::CSM_IssuerAndSerialNumber()
{
   m_pSNACCIssSn = NULL;
   m_pIssuer = NULL; 
   m_pSerialNo = NULL; 
}

// use this constructor to make a copy of the provided structure
// and put it into this structure
CSM_IssuerAndSerialNumber::CSM_IssuerAndSerialNumber(CSM_DN &dn, CSM_Buffer &serial)
{
   m_pSNACCIssSn = NULL;
   m_pIssuer = new CSM_DN(dn); 
   m_pSerialNo = new CSM_Buffer(serial); 
}

// use this constructor to make a new m_pSNACCIssSn SNACC structure
CSM_IssuerAndSerialNumber::CSM_IssuerAndSerialNumber(IssuerAndSerialNumber &SNACCIssSn)
{
   m_pIssuer = NULL; 
   m_pSerialNo = NULL; 
   m_pSNACCIssSn = new IssuerAndSerialNumber;
   *m_pSNACCIssSn = SNACCIssSn;
}

// use this constructor to get the issuer and serial number
// from a given certificate
CSM_IssuerAndSerialNumber::CSM_IssuerAndSerialNumber(CSM_Buffer *SNACCCertBuf) 
{
   Certificate tmpSNACCCert;

   m_pSNACCIssSn = NULL;
   m_pIssuer = NULL; 
   m_pSerialNo = NULL; 
   
   DECODE_BUF(&tmpSNACCCert, SNACCCertBuf);
   Set(tmpSNACCCert);
}

CSM_IssuerAndSerialNumber::CSM_IssuerAndSerialNumber(Certificate &SNACCCert)
{
   m_pSNACCIssSn = NULL;
   m_pIssuer = NULL; 
   m_pSerialNo = NULL; 
   Set(SNACCCert);
}

void CSM_IssuerAndSerialNumber::Set(Certificate &SNACCCert) 
{
   m_pIssuer = new CSM_DN(*SNACCCert.certificateToSign->issuer);
   if (m_pSerialNo == NULL)
   {    char *ptr=(char *)calloc(1, 
            SNACCCert.certificateToSign->serialNumber.Len());
        memcpy(ptr, (char *)SNACCCert.certificateToSign->serialNumber,
            SNACCCert.certificateToSign->serialNumber.Len());
        m_pSerialNo = new CSM_Buffer(ptr, // CAREFUL, buffer takes actual mem.
            SNACCCert.certificateToSign->serialNumber.Len()); 
   }
}

// returns a pointer to the CSM_DN class  
CSM_DN* CSM_IssuerAndSerialNumber::GetIssuer() 
{ 
   if ((m_pIssuer == NULL) && (m_pSNACCIssSn != NULL) &&
         (m_pSNACCIssSn->issuer != NULL))  
        m_pIssuer = new CSM_DN(*m_pSNACCIssSn->issuer);
   return m_pIssuer; 
}   

// returns a pointer to the CSM_Buffer class  
CSM_Buffer* CSM_IssuerAndSerialNumber::GetSerialNo() 
{ 
   if ((m_pSerialNo == NULL) && (m_pSNACCIssSn != NULL))  
      m_pSerialNo = new CSM_Buffer(m_pSNACCIssSn->serialNumber, 
         m_pSNACCIssSn->serialNumber.Len());
   return m_pSerialNo; 
}   

// set m_pIssuer private variable with a CSM_DN issuer
void CSM_IssuerAndSerialNumber::SetIssuer(CSM_DN &Issuer) 
{
   if (m_pSNACCIssSn != NULL)
      m_pSNACCIssSn = NULL;
   m_pIssuer = new CSM_DN(Issuer); 
}     

// set m_pIssuer private variable with a SNACC issuer
void CSM_IssuerAndSerialNumber::SetIssuer(Name &SnaccIssuer)
{
   if (m_pSNACCIssSn != NULL)
      m_pSNACCIssSn = NULL;
   m_pIssuer = new CSM_DN(SnaccIssuer); 
}     

// set m_pSerialNo private variable
void CSM_IssuerAndSerialNumber::SetSerialNo(CSM_Buffer &SerialNo) 
{
   if (m_pSNACCIssSn != NULL)
      m_pSNACCIssSn = NULL;
   m_pSerialNo = new CSM_Buffer(SerialNo); 
}

// COMPARISON OPERATORS
bool CSM_IssuerAndSerialNumber::operator == (CSM_IssuerAndSerialNumber &issuer) 
{
   // LOAD appropriate CSM member variables into memory from SNACC.
   GetIssuer();
   GetSerialNo();
   issuer.GetIssuer();
   issuer.GetSerialNo();

   if (m_pIssuer && m_pSerialNo) 
      return(*m_pIssuer == *issuer.m_pIssuer &&
         *m_pSerialNo == *issuer.m_pSerialNo);
   else return(0); 
}


CSM_IssuerAndSerialNumber & CSM_IssuerAndSerialNumber::operator = 
   (IssuerAndSerialNumber &SNACCISN) 
{
   if (m_pSNACCIssSn != NULL)
      delete m_pSNACCIssSn;
   m_pSNACCIssSn = new IssuerAndSerialNumber(SNACCISN);

   if (m_pIssuer != NULL)
   {
      delete m_pIssuer;
      m_pIssuer = NULL;
   }
   if (m_pSerialNo != NULL)
   {
      delete m_pSerialNo;
      m_pSerialNo = NULL;
   }
   return *this;
}

CSM_IssuerAndSerialNumber::operator IssuerAndSerialNumber* ()
{
   if (m_pSNACCIssSn == NULL)
      m_pSNACCIssSn = new IssuerAndSerialNumber;
   if (m_pSNACCIssSn->issuer && m_pIssuer)
      free(m_pSNACCIssSn->issuer);
   if (m_pIssuer)
      m_pSNACCIssSn->issuer = new Name;
      *m_pSNACCIssSn->issuer = *(Name*)*m_pIssuer;
   if (m_pSerialNo->Length())
      m_pSNACCIssSn->serialNumber.Set((char *)*m_pSerialNo, m_pSerialNo->Length());
   return(m_pSNACCIssSn);
}

void CSM_OID::PutChar(char *szOidCopy)
{
   unsigned long int aul[11];
   int i;
   char *ptr;

   SME_SETUP("CSM_OID::PutChar");

   // get the first value
   if ((ptr = strdup(szOidCopy)) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
   char *pszToken = strtok(ptr, ".");
   // fill array with -1
   for (i = 0; i < 11; i++) aul[i] = -1;
   // loop through all values
   i = 0;
   while (pszToken != NULL)
   {
      aul[i] = atol(pszToken);
      pszToken = strtok(NULL, ".");
      ++i;
   };
   Set(aul[0], aul[1], aul[2], aul[3], aul[4], aul[5], aul[6], 
         aul[7], aul[8], aul[9], aul[10]);
   if (ptr) 
       free(ptr);
   SME_FINISH_CATCH
}



char *CSM_OID::GetChar()
{
   unsigned short int firstArcNum;
   unsigned long int arcNum;
   unsigned int i;
   char szBuffer[128];
   SM_RET_VAL ret = SM_NO_ERROR;
   char *pOid;
   AsnOid *asnOid=this;

      const char *oid = asnOid->Str();
      size_t octetLen = asnOid->Len();

      // un-munge first two arc numbers
      for (arcNum = 0, i=0; (i < octetLen) && (oid[i] & 0x80); i++)
         arcNum = (arcNum << 7) + (oid[i] & 0x7f);

      arcNum = (arcNum << 7) + (oid[i] & 0x7f);
      i++;
      firstArcNum = (unsigned short int)arcNum/40;
      if (firstArcNum > 2)
         firstArcNum = 2;

      sprintf(szBuffer, "%ld.%ld", firstArcNum, (arcNum - (firstArcNum * 40)));

      for (; i < octetLen; )
      {
         for (arcNum = 0; (i < octetLen) && (oid[i] & 0x80); i++)
            arcNum = (arcNum << 7) + (oid[i] & 0x7f);

         arcNum = (arcNum << 7) + (oid[i] & 0x7f);
         i++;
         sprintf(szBuffer, "%s.%ld", szBuffer, arcNum);
      }
      pOid = strdup(szBuffer);

      return pOid;
}

//////////////////////////////////////////////////////////////////////////
// CSM_Recipient represents a single recipient for encrypting and 
// decrypting EnvelopedDatas.
void CSM_Recipient::Clear()
{
   m_bDecrypted = false;
   m_pRID = NULL;
   m_pOriginatorCert = NULL;
   m_pKeyEncryptionAlg = NULL;
}

//////////////////////////////////////////////////////////////////////////
void CSM_Recipient::SetFromSNACCRecipientInfo(RecipientInfo *pRI)
{
   SME_SETUP("CSM_Recipient::SetFromSNACCRecipientInfo");

   if (pRI)
   {
      if (pRI->rid)
      {
         switch (pRI->rid->choiceId)
         {
         case RecipientIdentifier::issuerAndSerialNumberCid:
            m_pRID = new CSM_IssuerAndSerialNumber
                  (*(pRI->rid->issuerAndSerialNumber));
            break;
         case RecipientIdentifier::rKeyIdCid:
            // TBD, not implemented
            break;
         case RecipientIdentifier::mlKeyIdCid:
            // TBD, not implemented
            break;
         }
      }
      if (pRI->originatorCert)
      {
         m_pOriginatorCert = new CSM_Entity(*(pRI->originatorCert));
      }
      if (pRI->keyEncryptionAlgorithm)
      {
         m_pKeyEncryptionAlg = new CSM_Alg(*(pRI->keyEncryptionAlgorithm));
      }
   }

   SME_FINISH_CATCH
}

//////////////////////////////////////////////////////////////////////////
CSM_Entity::CSM_Entity(EntityIdentifier &ei)
{
   SME_SETUP("CSM_EntityIdentifier::CSM_EntityIdentifier(snacc)");

   if (ei.choiceId == EntityIdentifier::issuerAndSerialNumberCid)
   {
      m_lIdType = ENTITY_ISSUER_SERIAL;
      SME(m_pIssuerSerial = new CSM_IssuerAndSerialNumber(*(ei.
            issuerAndSerialNumber)));
   }
   else // assume EntityIdentifier::subjectKeyIdentifier
   {
      m_lIdType = ENTITY_KEY_ID;
      SME(m_pKeyId = new CSM_Buffer((char*)(*ei.subjectKeyIdentifier),
               ei.subjectKeyIdentifier->Len()));
   }

   SME_FINISH_CATCH
}

// EOF sm_CsmSupport.cpp
