#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_MsgCertCrls.cpp 1.4 3/25/98 10:28:12"; 
#endif

//////////////////////////////////////////////////////////////////////////
// sm_MsgCertCrls.cpp
// methods for the CSM_MsgCertCrls class

#include "sm_api.h"

//////////////////////////////////////////////////////////////////////////
CSM_MsgCertCrls::~CSM_MsgCertCrls()
{
   if (m_pCerts)
      delete (m_pCerts);
   if (m_pACs)
      delete (m_pACs);
   if (m_pCRLs)
      delete (m_pCRLs);
}

//////////////////////////////////////////////////////////////////////////
void CSM_MsgCertCrls::ClearCerts()
{
   if (m_pCerts)
   {
      delete (m_pCerts);
      m_pCerts = NULL;
   }
}

//////////////////////////////////////////////////////////////////////////
void CSM_MsgCertCrls::ClearACs()
{
   if (m_pACs)
   {
      delete (m_pACs);
      m_pACs = NULL;
   }
}

//////////////////////////////////////////////////////////////////////////
void CSM_MsgCertCrls::ClearCRLs()
{
   if (m_pCRLs)
   {
      delete (m_pCRLs);
      m_pCRLs = NULL;
   }
}

//////////////////////////////////////////////////////////////////////////
// SetCertificates COPIES the Buffers from the provided buffer list into
// a list of cert choice classes.  In other words, each buffer from the
// provided list is duplicated and added to or placed into a new list
// of cert choice classes.  Therefore, the caller can delete the provided
// buffer list after SetCertificates returns
void CSM_MsgCertCrls::SetCertificates(CSM_BufferLst *pCerts)
{
   CSM_BufferLst *pTmpBufferLst;
   CSM_CertificateChoice *pTmpNewCert;
   CSM_Buffer *pNewBuffer;

   SME_SETUP("CSM_MsgCertCrls::SetCertificates(CSM_BufferLst*)");

   if (pCerts)
   {
      pTmpBufferLst = pCerts->First();
      do
      {
         // copy the cert from the provided list
         if ((pNewBuffer = new CSM_Buffer(*(pTmpBufferLst->Get()))) == NULL)
            SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
         // place the new buffer (cert) into a CSM_CertificateChoice
         if ((pTmpNewCert = new CSM_CertificateChoice(pNewBuffer)) == NULL)
            SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
         // either create a new list of CSM_CertificateChoices
         if (m_pCerts == NULL)
         {
            if ((m_pCerts = new CSM_CertificateChoiceLst(pTmpNewCert)) 
                  == NULL)
               SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
         }
         // or add the cert choice to the end of the current list
         else
            m_pCerts->Append(pTmpNewCert);
      } while ((pTmpBufferLst = pTmpBufferLst->Next()) != NULL);
   }
   SME_FINISH
   SME_CATCH_SETUP
      // local cleanup logic
   SME_CATCH_FINISH
}

//////////////////////////////////////////////////////////////////////////
// SetCertificates COPIES the CertChoices from the provided list into
// the right list of cert choice classes.  In other words, each cert choice
// from the provided list is duplicated and added or placed into a new
// list of cert choice classes depending on the type of the given
// cert choice (cert or AC), therefore, the caller can delete the provided
// cert choice list after SetCertificates returns
void CSM_MsgCertCrls::SetCertificates(CSM_CertificateChoiceLst *pCerts)
{
   CSM_CertificateChoiceLst *pTmpCertChoiceLst;
   CSM_CertificateChoice *pTmpNewCert;

   SME_SETUP("CSM_MsgCertCrls::SetCertificates(CSM_CertificateChoiceLst*)");

   if (pCerts)
   {
      pTmpCertChoiceLst = pCerts->First();
      do
      {
         // copy the cert from the provided list
         if ((pTmpNewCert = new CSM_CertificateChoice(*(pTmpCertChoiceLst->
                  Get()))) == NULL)
            SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
         // depending on the type, add it accordingly
         if (pTmpNewCert->m_lCertType == pTmpNewCert->CERT_TYPE_CERT)
         {
            // either create a new list of CSM_CertificateChoices
            if (m_pCerts == NULL)
            {
               if ((m_pCerts = new CSM_CertificateChoiceLst(pTmpNewCert)) 
                     == NULL)
                  SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
            }
            // or add the cert choice to the end of the current list
            else
               m_pCerts->Append(pTmpNewCert);
         }
         else // m_lCertType == CERT_TYPE_ATTR_CERT
         {
            // either create a new list of CSM_CertificateChoices
            if (m_pACs == NULL)
            {
               if ((m_pACs = new CSM_CertificateChoiceLst(pTmpNewCert)) 
                     == NULL)
                  SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
            }
            // or add the cert choice to the end of the current list
            else
               m_pACs->Append(pTmpNewCert);
         }
      } while ((pTmpCertChoiceLst = pTmpCertChoiceLst->Next()) != NULL);
   }
   SME_FINISH
   SME_CATCH_SETUP
      // local cleanup logic
   SME_CATCH_FINISH
}

//////////////////////////////////////////////////////////////////////////
void CSM_MsgCertCrls::SetCRLs(CSM_BufferLst *pCRLs)
{
   CSM_BufferLst *pTmpBufferLst;
   CSM_Buffer *pNewBuffer;

   SME_SETUP("CSM_MsgCertCrls::SetCRLs");

   if (pCRLs)
   {
      pTmpBufferLst = pCRLs->First();
      do
      {
         // copy the CRL from the provided list
         if ((pNewBuffer = new CSM_Buffer(*(pTmpBufferLst->Get()))) == NULL)
            SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
         // either create a new list of CSM_Buffers
         if (m_pCRLs == NULL)
         {
            if ((m_pCRLs = new CSM_BufferLst(pNewBuffer)) 
                  == NULL)
               SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
         }
         // or add the CRL buffer to the end of the current list
         else
            m_pCRLs->Append(pNewBuffer);
      } while ((pTmpBufferLst = pTmpBufferLst->Next()) != NULL);
   }
   SME_FINISH
   SME_CATCH_SETUP
      // local cleanup logic
   SME_CATCH_FINISH
}

//////////////////////////////////////////////////////////////////////////
// AddCert adds the provided cert to the m_pCerts or m_pACs according
// to the type of the provided cert.  The pointer is copied in so the
// caller should NOT free the pCert after AddCert returns...
void CSM_MsgCertCrls::AddCert(CSM_CertificateChoice *pCert)
{
   SME_SETUP("CSM_MsgCertCrls::AddCert");

   if (pCert)
   {
      // depending on the type, add it accordingly
      if (pCert->m_lCertType == pCert->CERT_TYPE_CERT)
      {
         // either create a new list of CSM_CertificateChoices
         if (m_pCerts == NULL)
         {
            if ((m_pCerts = new CSM_CertificateChoiceLst(pCert)) 
                  == NULL)
               SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
         }
         // or add the cert choice to the end of the current list
         else
            m_pCerts->Append(pCert);
      }
      else // m_lCertType == CERT_TYPE_ATTR_CERT
      {
         // either create a new list of CSM_CertificateChoices
         if (m_pACs == NULL)
         {
            if ((m_pACs = new CSM_CertificateChoiceLst(pCert)) 
                  == NULL)
               SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
         }
         // or add the cert choice to the end of the current list
         else
            m_pACs->Append(pCert);
      }
   }
   SME_FINISH
   SME_CATCH_SETUP
      // local cleanup logic
   SME_CATCH_FINISH
}

//////////////////////////////////////////////////////////////////////////
// PutSNACCCerts ASN.1 decodes the m_pCerts and returns them in the
// provided pCertificateSet
void CSM_MsgCertCrls::PutSNACCCerts(CertificateSet *&pCertificateSet)
{
   CSM_CertificateChoiceLst *tmpCertList=NULL;
   CertificateChoices *tmnSNACCCertChoices=NULL;

   SME_SETUP("CSM_MsgCertCrls::PutSNACCCerts");

   if (pCertificateSet == NULL)    // initialize for application
      if ((pCertificateSet = new CertificateSet) == NULL)
            SME_THROW(SM_MEMORY_ERROR, 
                  "SNACC Certificate bad memory alloc", NULL);

   if (pCertificateSet)
   {
      // we can only do this if we have ASN.1 encoded certs
      if (m_pCerts)
      {
         // loop through the m_pCerts here
         for (tmpCertList=m_pCerts->First(); tmpCertList; 
               tmpCertList=tmpCertList->Next())
         { // For each cert buffer, load into SNACC cert list.
            tmnSNACCCertChoices = pCertificateSet->Append();

            // do a regular cert
            if (tmpCertList->Get()->m_lCertType == 
                  CSM_CertificateChoice::CERT_TYPE_CERT)
            {
               // set the type to cert
               tmnSNACCCertChoices->choiceId = 
                     CertificateChoices::certificateCid;
               // allocate a snacc cert class
               if ((tmnSNACCCertChoices->certificate = new Certificate) 
                        == NULL)
                  SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
               DECODE_BUF(tmnSNACCCertChoices->certificate, 
                     (*(tmpCertList->Get()->GetEncodedCert())));
            }
            // do an attribute cert
            else if (tmpCertList->Get()->m_lCertType == 
                  CSM_CertificateChoice::CERT_TYPE_ATTR_CERT)
            {
               // set the type to attribute cert
               tmnSNACCCertChoices->choiceId = 
                     CertificateChoices::attrCertCid;
               // allocate a snacc attribute cert class
               if ((tmnSNACCCertChoices->attrCert = new AttributeCertificate)
                        == NULL)
                  SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
               DECODE_BUF(tmnSNACCCertChoices->attrCert, 
                     (*(tmpCertList->Get()->GetEncodedCert())));
            }
            // extended cert not handled
         }
      }
   }

   SME_FINISH
   SME_CATCH_SETUP
      // local cleanup logic
   SME_CATCH_FINISH
}

//////////////////////////////////////////////////////////////////////////
// SetSNACCCerts places the provided snacc decoded certs
// into this appropriately
void CSM_MsgCertCrls::SetSNACCCerts(CertificateSet *pCertificateSet)
{
   CertificateChoices *pTmpSNACCCert;
   CSM_CertificateChoice *pCertChoice;

   SME_SETUP("CSM_MsgCertCrls::SetSNACCCerts");

   if (pCertificateSet == NULL)
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   for (pCertificateSet->SetCurrToFirst(),
         pTmpSNACCCert = pCertificateSet->Curr();
         pTmpSNACCCert;
         pTmpSNACCCert = pCertificateSet->GoNext())
   {
      if ((pCertChoice = new CSM_CertificateChoice(*pTmpSNACCCert)) == NULL)
         SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
      SME(AddCert(pCertChoice));
   }
   SME_FINISH
   SME_CATCH_SETUP
      // local cleanup logic
   SME_CATCH_FINISH
}

//////////////////////////////////////////////////////////////////////////
// SetSNACCCrls places the provided snacc decoded crls
// into this appropriately
void CSM_MsgCertCrls::SetSNACCCrls(CertificateRevocationLists *pCrls)
{
   CertificateList *pTmpSNACCCRL;
   CSM_Buffer *pbuf;

   SME_SETUP("CSM_MsgCertCrls::SetSNACCCrls");

   if (pCrls == NULL)
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   for (pCrls->SetCurrToFirst(), pTmpSNACCCRL = pCrls->Curr();
         pTmpSNACCCRL; pTmpSNACCCRL = pCrls->GoNext())
   {
      ENCODE_BUF(pTmpSNACCCRL, pbuf);
      if (m_pCRLs == NULL)
      {
         if ((m_pCRLs = new CSM_BufferLst(pbuf)) == NULL)
            SME_THROW(SM_MEMORY_ERROR, NULL, NULL)
      }
      else
         m_pCRLs->Append(pbuf);
   }

   SME_FINISH_CATCH
}

// EOF sm_MsgCertCrls.cpp
