#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_buffer.cpp 1.12 3/27/98 18:33:02"; 
#endif

//////////////////////////////////////////////////////////////////////////
// sm_buffer.cpp
// This source file implements various members of the CSM_Buffer class.
// Be careful when you modify these
// members because code is being written based on the characteristics
// of these members...
//////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#ifdef SUNOS     
#include <unistd.h>   // for SEEK_CUR and SEEK_END
#endif

#include "sm_api.h"
#include "sm_internal.h"

//////////////////////////////////////////////////////////////////////////
CSM_Buffer::CSM_Buffer()
{
   m_pMemory = m_pszFN = m_pMemFP = NULL;
   m_pFP = NULL;
   SetLength(0);
   m_pCache = NULL;
   m_lCacheSize = 0;
}

//////////////////////////////////////////////////////////////////////////
CSM_Buffer::CSM_Buffer(size_t lSize)
{ 
   SME_SETUP("CSM_Buffer::CSM_Buffer(size_t)");
   m_pszFN = m_pMemFP = NULL;
   m_pFP = NULL;
   if ((m_pMemory = (char *)calloc(1, lSize + 1)) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
   SME(SetLength(lSize));
   m_pCache = NULL;
   m_lCacheSize = 0;
   SME_FINISH_CATCH
}

//////////////////////////////////////////////////////////////////////////
CSM_Buffer::CSM_Buffer(char *pszFileName)
{
   SME_SETUP("CSM_Buffer::CSM_Buffer(char*)");
   if (pszFileName == NULL)
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);
   m_pMemory = m_pMemFP = NULL;
   m_pFP = NULL;
   SME(SetLength(0));
   if ((m_pszFN = strdup(pszFileName)) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
   m_pCache = NULL;
   m_lCacheSize = 0;
   SME_FINISH_CATCH
}

//////////////////////////////////////////////////////////////////////////
CSM_Buffer::CSM_Buffer(char *pBuf, SM_SIZE_T lSize) 
{
   SME_SETUP("CSM_Buffer::CSM_Buffer(char *, size_t)");
   if (pBuf == NULL)
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);
   m_pszFN = m_pMemFP = NULL;
   m_pFP = NULL;
   SME(Set(pBuf, lSize));
   m_pCache = NULL;
   m_lCacheSize = 0;
   SME_FINISH_CATCH
}

//////////////////////////////////////////////////////////////////////////
CSM_Buffer::CSM_Buffer(CSM_Buffer &b) 
{
   m_pMemory = m_pszFN = m_pMemFP = NULL;
   m_pFP = NULL;
   SetLength(0);
   m_pCache = NULL;
   m_lCacheSize = 0;
   ReSet(b);
}

//////////////////////////////////////////////////////////////////////////
CSM_Buffer::~CSM_Buffer()
{
   if (m_pMemory)
      free (m_pMemory);
   if (m_pszFN)
      free (m_pszFN);
   if (m_pFP)
      fclose(m_pFP);
   if (m_pCache)
      free (m_pCache);
}

//////////////////////////////////////////////////////////////////////////
SM_SIZE_T CSM_Buffer::Length()
{
   SM_SIZE_T lRet = 0;

   SME_SETUP("CSM_Buffer::Length");

   if (InFile())
   {
      // file version
      struct stat statBuf;
      // how big is data in file
      if (stat(m_pszFN, &statBuf) == -1)
      {
         char szMsg[512];
         sprintf(szMsg, "Couldn't stat file %s", m_pszFN);
         SME_THROW(SM_FILEIO_ERROR, szMsg, NULL);
      }
      lRet = statBuf.st_size;
   }
   else
   {
      // memory version
      lRet = m_lSize;
   }

   SME_FINISH_CATCH

   return lRet;
}

//////////////////////////////////////////////////////////////////////////
void CSM_Buffer::Set(char *psz)
{
   SME_SETUP("CSM_Buffer::Set(char *)");
   if (psz == NULL)
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);
   if ((m_pMemory = strdup(psz)) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
   SME(SetLength(strlen(psz)));
   SME_FINISH_CATCH
}

//////////////////////////////////////////////////////////////////////////
void CSM_Buffer::Set(char *p, SM_SIZE_T lSize)
{
   SME_SETUP("CSM_Buffer::Set(char *, size_t)");
   if (p == NULL)
   {
      m_pMemory = NULL;
      SME(SetLength(0));
   }
   else
   {
      if ((m_pMemory = (char *)calloc(1, lSize + 1)) == NULL)
         SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
      memcpy(m_pMemory, p, lSize);
      SME(SetLength(lSize));
   }
   SME_FINISH_CATCH
}

//////////////////////////////////////////////////////////////////////////
// allocate memory in the cache
char* CSM_Buffer::Alloc(SM_SIZE_T lSize)
{
   SME_SETUP("CSM_Buffer::Alloc");

   if ((m_pCache = (char *)calloc(1, lSize)) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
   m_lCacheSize = lSize;

   SME_FINISH_CATCH

   return m_pCache;
}

//////////////////////////////////////////////////////////////////////////
void CSM_Buffer::AllocMoreMem(SM_SIZE_T lSize)
{
   char *pNew;
   SM_SIZE_T lLength = Length();

   SME_SETUP("CSM_Buffer::AllocMoreMem");

   if ((pNew = (char *)calloc(1, lLength + lSize)) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
   memcpy(pNew, m_pMemory, lLength);
   SetLength(lLength + lSize);
   m_pMemFP = pNew + (m_pMemFP - m_pMemory);
   free(m_pMemory);
   m_pMemory = pNew;

   SME_FINISH_CATCH
}

//////////////////////////////////////////////////////////////////////////
char* CSM_Buffer::AccessAll()
{
   SME_SETUP("CSM_Buffer::AccessAll");
   if (InFile())
   {
      // if the data is in a file AND
      // if there's already memory in m_pMemory then free it
      if (m_pMemory != NULL)
         free (m_pMemory);
      SME(m_pMemory = CopyAll());
   }
   SME_FINISH_CATCH
   return m_pMemory;
}

//////////////////////////////////////////////////////////////////////////
// return a copy of the actual data and return the size
char* CSM_Buffer::CopyAll(SM_SIZE_T &l)
{
   char *pRet = NULL;
   SM_SIZE_T lSize = Length();
   SME_SETUP("CSM_Buffer::CopyAll");

   if (InFile()) // data in file
   {
      // allocate memory
      if ((pRet = (char *)calloc(1, lSize + 1)) == NULL)
         SME_THROW(SM_MEMORY_ERROR, "calloc failure", NULL);
      // close file if present
      if (m_pFP != NULL)
         fclose(m_pFP);
      // open the file
      if ((m_pFP = fopen(m_pszFN, SM_FOPEN_READ)) == NULL)
      {
         char szMsg[512];
         sprintf(szMsg, "Couldn't open file %s", m_pszFN);
         SME_THROW(SM_FILEIO_ERROR, szMsg, NULL);
      }
      // read the data
      long lRead = fread(pRet, 1, lSize, m_pFP);
      if (ferror(m_pFP) != 0)
      {
         char szMsg[512];
         sprintf(szMsg, "Couldn't read file %s", m_pszFN);
         SME_THROW(SM_FILEIO_ERROR, szMsg, NULL);
      }
      // close and clear FP
      fclose(m_pFP);
      m_pFP = NULL;
      l = lRead; // store the size that will be returned
   }
   else
   {
      // dup the data
      pRet = (char *)calloc(1, lSize);
      memcpy(pRet, m_pMemory, lSize);
      l = lSize; // store the size that will be returned
   }

   SME_FINISH
   SME_CATCH_SETUP
      if (pRet != NULL)
         free(pRet);
      if (m_pFP != NULL)
      {
         fclose(m_pFP);
         m_pFP = NULL;
      }
   SME_CATCH_FINISH
   return pRet;
}

//////////////////////////////////////////////////////////////////////////
// compare buffers regardless of memory/file status
long CSM_Buffer::Compare(/*const*/ CSM_Buffer &b)
{
   char *p1 = NULL;
   char *p2 = NULL;
   long lRet = -2;

   SME_SETUP("CSM_Buffer::Compare");
   // use AccessAll on both buffers for comparison.  If buffer is in
   // file, then this results in a CopyAll which isn't as efficient,
   // but this can be fixed later...
   if ((p1 = AccessAll()) != NULL)
   {
      if ((p2 = b.AccessAll()) != NULL)
      {
         if (Length() == b.Length())
            lRet = (long)memcmp(p1, p2, Length());
         if (InFile()) 
            free (p1);
         if (b.InFile()) 
            free (p2);
      }
      else
         if (InFile()) 
            free (p1);
   }
   SME_FINISH_CATCH
   return lRet;
}

//////////////////////////////////////////////////////////////////////////
// copy b into this
SM_RET_VAL CSM_Buffer::ReSet(CSM_Buffer &b)
{
   char *p;
   SM_SIZE_T l;
   SME_SETUP("CSM_Buffer::ReSet");

   m_pMemory = m_pszFN = m_pMemFP = NULL;
   m_pFP = NULL;
   SME(SetLength(0));
   m_pCache = NULL;
   m_lCacheSize = 0;
   
   SME(p = b.CopyAll(l));

   SME(Set(p, l));

   free(p);

   SME_FINISH_CATCH

   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
// ConvertFileToMemory makes a CSM_Buffer storing its contents in
// file into a CSM_Buffer storing its contents in memory
SM_RET_VAL CSM_Buffer::ConvertFileToMemory()
{
   SM_SIZE_T l;

   SME_SETUP("CSM_Buffer::ConvertFileToMemory");

   if (m_pszFN == NULL)
      // we're already in memory
      return SM_NO_ERROR;

   // read everything into memory
   SME(m_pMemory = CopyAll(l));

   // free the file name
   free(m_pszFN);
   m_pszFN = NULL;

   // store the new size
   SME(SetLength(l));

   SME_FINISH_CATCH

   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
// ConvertMemoryToFile makes a CSM_Buffer storing its contents in
// buffer into a CSM_Buffer storing its contents in file
SM_RET_VAL CSM_Buffer::ConvertMemoryToFile(char *pszFN)
{
   SM_SIZE_T lRet = 0;

   SME_SETUP("CSM_Buffer::ConvertMemoryToFile");

   if (pszFN == NULL)
      SME_THROW(SM_NO_FILENAME, NULL, NULL);

   if (InFile())
   {
     if (strcmp(m_pszFN, pszFN) == 0)   // we're already in file
        return SM_NO_ERROR;
     else
     {
        SM_SIZE_T lBytesRead;
        SM_SIZE_T lSize=4096;
        char *ptr;
        FILE *fp=fopen(pszFN, "w");
        this->Open(SM_FOPEN_READ);
        while ((ptr=this->nRead(lSize, lBytesRead)) != NULL && lBytesRead > 0)
        {
            fwrite(ptr, 1, lBytesRead, fp);
        }
        this->Close();
        fclose(fp);
        return(SM_NO_ERROR);
     }
   }

   // open the new file
   if ((m_pFP = fopen(pszFN, SM_FOPEN_WRITE)) == NULL)
   {
      char szMsg[512];
      sprintf(szMsg, "Couldn't stat file %s", pszFN);
      SME_THROW(SM_FILEIO_ERROR, szMsg, NULL);
   }

   // write the data
   SM_SIZE_T lLength = Length();
   // store the file name
   if ((m_pszFN = strdup(pszFN)) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

   if ((lRet = fwrite(m_pMemory, 1, lLength, m_pFP)) != lLength)
   {
      char szMsg[512];
      sprintf(szMsg, "Couldn't write file %s", m_pszFN);
      SME_THROW(SM_FILEIO_ERROR, szMsg, NULL);
   }

   fclose(m_pFP);
   m_pFP = NULL;

   SME_FINISH
   SME_CATCH_SETUP
      // cleanup/catch code
      if ((m_pszFN != NULL) && (pszFN != NULL))
      {
         free(m_pszFN);
         m_pszFN = NULL;
      }
   SME_CATCH_FINISH

   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL CSM_Buffer::Open(char *pszMode)
{
   SME_SETUP("CSM_Buffer::Open");

   if (pszMode == NULL)
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   if (!InFile())
      // memory version
      m_pMemFP = m_pMemory; // set current pointer to start
   else
      // file version
      if ((m_pFP = fopen(m_pszFN, pszMode)) == NULL)
      {
         char szMsg[512];
         sprintf(szMsg, "Couldn't open file %s", m_pszFN);
         SME_THROW(SM_FILEIO_ERROR, szMsg, NULL);
      }

   SME_FINISH_CATCH
   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL CSM_Buffer::Seek(SM_SIZE_T lOffset, SM_SIZE_T lOrigin)
{
   SM_RET_VAL lRet = SM_NO_ERROR;

   SME_SETUP("CSM_Buffer::Seek");

   if (!InFile())
   {
      // memory version
      char *pSave = m_pMemFP;

      if (m_pMemFP == NULL)
         SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

      SM_SIZE_T lLength = Length();

      switch (lOrigin)
      {
      case SEEK_CUR:
         m_pMemFP += lOffset;
         break;
      case SEEK_END:
         m_pMemFP = (m_pMemory + lLength - 1) + lOffset;
         break;
      default: // SEEK_SET
         m_pMemFP = m_pMemory + lOffset;
         break;
      }
      if ((m_pMemFP > (m_pMemory + lLength - 1)) ||
          (m_pMemFP < m_pMemory))
      {
         m_pMemFP = pSave;
         lRet = -1;
      }
   }
   else
   {
      // file version
      if (m_pFP == NULL)
         SME_THROW(SM_FILEIO_ERROR, "FP is NULL", NULL);

      lRet = fseek(m_pFP, lOffset, lOrigin);
   }

   SME_FINISH_CATCH

   return lRet;
}

//////////////////////////////////////////////////////////////////////////
void CSM_Buffer::Close()
{
   if (m_pFP != NULL)
   {
      fclose(m_pFP);
      m_pFP = NULL;
      if (m_pMemory)
      {
         free(m_pMemory);
         m_pMemory = NULL;
      }
   }
   else
      m_pMemFP = NULL;
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL CSM_Buffer::cRead(char *pBuffer, SM_SIZE_T lSize)
{
   SM_RET_VAL lRet = 0;

   SME_SETUP("CSM_Buffer::cRead");

   if ((pBuffer == NULL) || (lSize <= 0))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   if (!InFile())
   {
      // memory version
      if (m_pMemFP == NULL)
         SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

      SM_SIZE_T lReadSize = lSize;
      SM_SIZE_T lLength = Length();
      // adjust the read size to what's possible
      if ((m_pMemFP + lReadSize) > (m_pMemory + lLength))
         lReadSize = (m_pMemory + lLength) - m_pMemFP;
      memcpy(pBuffer, m_pMemFP, lReadSize);
      // adjust the current pointer
      if (lReadSize > 0)
      {
         m_pMemFP += lReadSize;
         lRet = lReadSize;
      }
      else
         lRet = 0;
   }
   else
   {
      // file version
      if (m_pFP == NULL)
         SME_THROW(SM_FILEIO_ERROR, "FP is NULL", NULL);

      lRet = fread(pBuffer, 1, lSize, m_pFP);
   }

   SME_FINISH_CATCH

   return lRet;
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL CSM_Buffer::Write(char *pBuffer, SM_SIZE_T lSize)
{
   SM_RET_VAL lRet = 0;

   SME_SETUP("CSM_Buffer::Write");

   if ((pBuffer == NULL) || (lSize <= 0))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   if (!InFile())
   {
      // memory version
      if (m_pMemFP == NULL)
      {
         if (m_pMemory == NULL)
         {
            // if we get here, we assume that the memory
            // hasn't been allocated yet, allocate it...
            if ((m_pMemFP = m_pMemory = (char *)calloc(1, lSize)) == NULL)
               SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
            SetLength(lSize);
         }
         else
            m_pMemFP = m_pMemory;
      }

      // do we have enough space to write to this buffer?
      if ((SM_SIZE_T)(((m_pMemory + Length()) - m_pMemFP)) < lSize)
         // nope, get lSize more bytes
         AllocMoreMem(lSize);
      memcpy(m_pMemFP, pBuffer, lSize);
      m_pMemFP += lSize;
      lRet = lSize;
   }
   else
   {
      // file version
      if (m_pFP == NULL)
         SME_THROW(SM_FILEIO_ERROR, "FP is NULL", NULL);

      if ((lRet = fwrite(pBuffer, 1, lSize, m_pFP)) > 0)
         SetLength(m_lSize + lRet);
   }

   SME_FINISH_CATCH

   return lRet;
}

//////////////////////////////////////////////////////////////////////////
char* CSM_Buffer::nRead(SM_SIZE_T lSize, SM_SIZE_T &lBytesRead)
{
   char *pRet = NULL;

   SME_SETUP("CSM_Buffer::nRead");

   if (lSize <= 0)
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   if (!InFile())
   {
      // memory version
      if (m_pMemFP == NULL)
         SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

      SM_SIZE_T lReadSize = lSize;
      SM_SIZE_T lLength = Length();
      // adjust the read size to what's possible
      if ((m_pMemFP + lReadSize) > (m_pMemory + lLength))
         lReadSize = (m_pMemory + lLength) - m_pMemFP;
      pRet = m_pMemFP;
      // adjust the current pointer
      if (lReadSize > 0)
      {
         m_pMemFP += lReadSize;
         lBytesRead = lReadSize;
      }
      else
         lBytesRead = 0;
   }
   else
   {
      // file version
      if (m_pFP == NULL)
         SME_THROW(SM_FILEIO_ERROR, "FP is NULL", NULL);
      // if there's something already in the memory, free it
      if (m_pMemory != NULL)
         free (m_pMemory);
      // allocate memory to receive the read data
      if ((m_pMemory = (char *)calloc(1, lSize + 1)) == NULL)
         SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
      // now, read into the memory cache
      lBytesRead = fread(m_pMemory, 1, lSize, m_pFP);
      // now set what we'll return
      pRet = m_pMemory;
   }

   SME_FINISH_CATCH

   return pRet;
}

//////////////////////////////////////////////////////////////////////////
void CSM_Buffer::Flush()
{
   if (m_pCache != NULL)
   {
      Write(m_pCache, m_lCacheSize);
      free(m_pCache);
      m_pCache = NULL;
      m_lCacheSize = 0;
   }
}

// EOF sm_buffer.cpp
