#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_certChoice.cpp 1.7 3/25/98 12:12:52"; 
#endif

//////////////////////////////////////////////////////////////////////////
// sm_certChoice.cpp
// methods for the CSM_CertificateChoice class

#include "sm_api.h"

//////////////////////////////////////////////////////////////////////////
// clear method
void CSM_CertificateChoice::Clear()
{
   m_pKeyOID = NULL;
   m_pSubject = NULL;
   m_pIssuer = NULL;
   m_pCert = NULL;
   m_pSnaccCert = NULL;
   m_pSnaccAttr = NULL;
   m_lCertType = CERT_TYPE_CERT;
}

//////////////////////////////////////////////////////////////////////////
CSM_CertificateChoice::CSM_CertificateChoice(CSM_Buffer *pCert)
{
   Clear();
   SetEncodedCert(pCert);
}

//////////////////////////////////////////////////////////////////////////
CSM_CertificateChoice::CSM_CertificateChoice(CSM_Buffer *pCert, long lType)
{
   Clear();
   SetEncodedCert(pCert, lType);
}

//////////////////////////////////////////////////////////////////////////
CSM_CertificateChoice::CSM_CertificateChoice(CertificateChoices &SNACCCertChoices)
{
   Clear();
   SetSNACCCertChoices(SNACCCertChoices);
}

//////////////////////////////////////////////////////////////////////////
CSM_CertificateChoice::CSM_CertificateChoice(CSM_CertificateChoice &certChoice)
{
   Clear();
   m_pCert = new CSM_Buffer(*(certChoice.GetEncodedCert()));
   m_lCertType = certChoice.m_lCertType;
}

//////////////////////////////////////////////////////////////////////////
CSM_CertificateChoice::~CSM_CertificateChoice()
{
   // free memory if necessary
   if (m_pCert != NULL)
      delete m_pCert;
   if (m_pKeyOID != NULL)
      delete m_pKeyOID;
   if (m_pSubject != NULL)
      delete m_pSubject;
   if (m_pIssuer != NULL)
      delete m_pIssuer;
   if (m_pSnaccCert != NULL)
      delete m_pSnaccCert;
   if (m_pSnaccAttr != NULL)
      delete m_pSnaccAttr;
}

//////////////////////////////////////////////////////////////////////////
void CSM_CertificateChoice::SetEncodedCert(CSM_Buffer *pCert)
{
   if (pCert == NULL)
      return;
   // free memory if necessary
   if (m_pCert != NULL)
      delete m_pCert;
   if (m_pKeyOID != NULL)
      delete m_pKeyOID;
   if (m_pSubject != NULL)
      delete m_pSubject;
   if (m_pIssuer != NULL)
      delete m_pIssuer;
   if (m_pSnaccCert != NULL)
      delete m_pSnaccCert;
   if (m_pSnaccAttr != NULL)
      delete m_pSnaccAttr;
   // clear members
   Clear();
   // set recipient
   m_pCert = new CSM_Buffer(*pCert);
   m_lCertType = CERT_TYPE_CERT;
}

//////////////////////////////////////////////////////////////////////////
void CSM_CertificateChoice::SetEncodedCert(CSM_Buffer *pCert, long lType)
{
   SetEncodedCert(pCert);
   m_lCertType = lType;
}

//////////////////////////////////////////////////////////////////////////
void CSM_CertificateChoice::Decode()
{
   SME_SETUP("CSM_CertificateChoice::Decode");

   // error if it isn't present
   if (m_pCert == NULL)
      SME_THROW(SM_NO_CERT_SET, "encoded cert missing", NULL);

   // TBD, this needs to be updated to handle attribute certificates...
   if (m_lCertType != CERT_TYPE_CERT)
      SME_THROW(SM_NO_CERT_SET, "no support for attrib certs", NULL);

   if ((m_pSnaccCert = new Certificate) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

   // decode the certificate
   DECODE_BUF(m_pSnaccCert, m_pCert);

   if (m_pSnaccCert->certificateToSign == NULL)
      SME_THROW(SM_CERT_DEC_ERROR, "Certificate missing CTS", NULL);
   if (m_pSnaccCert->certificateToSign->subjectPublicKeyInfo == NULL)
      SME_THROW(SM_CERT_DEC_ERROR, "Certificate missing subj pub key info",
            NULL);

   // store Key OID
   if ((m_pKeyOID = new CSM_OID(m_pSnaccCert->certificateToSign->
         subjectPublicKeyInfo->algorithm->algorithm)) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

   // store issuer
   if (m_pSnaccCert->certificateToSign->issuer == NULL)
      SME_THROW(SM_CERT_DEC_ERROR, "Certificate missing issuer", NULL);
   if ((m_pIssuer = new CSM_DN(*(m_pSnaccCert->certificateToSign->issuer))) 
         == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

   // store subject
   if (m_pSnaccCert->certificateToSign->subject == NULL)
      SME_THROW(SM_CERT_DEC_ERROR, "Certificate missing subject", NULL);
   if ((m_pSubject = new CSM_DN(*(m_pSnaccCert->certificateToSign->subject)))
         == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

   SME_FINISH_CATCH;
}

//////////////////////////////////////////////////////////////////////////
CSM_OID* CSM_CertificateChoice::GetKeyOID()
{
   SME_SETUP("CSM_CertificateChoice::GetKeyOID");

   // if m_pKeyOID is null, then the cert has not been decoded
   // and we must decode it before we can return the requested value
   if (m_pKeyOID == NULL)
      SME(Decode());

   SME_FINISH_CATCH;
   return m_pKeyOID;
}

//////////////////////////////////////////////////////////////////////////
// GetPublicKey return's the cert's public key in a buffer
CSM_Buffer* CSM_CertificateChoice::GetPublicKey()
{
   CSM_Buffer *pRet = NULL;

   SME_SETUP("CSM_CertificateChoice::GetPublicKey");

   // if m_pSnaccCert is null then the cert has not been decoded
   // and we must decode it before we can return the requested value
   if (m_pSnaccCert == NULL)
      SME(Decode());

   AsnBits *pY = &(m_pSnaccCert->certificateToSign->subjectPublicKeyInfo->
         subjectPublicKey);
   if ((pRet = new CSM_Buffer) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
   if (SM_AsnBits2Buffer(pY, pRet) != SM_NO_ERROR)
      SME_THROW(SM_BAD_PUBLIC_VALUE, 
            "cannot convert cert's public value", NULL);
   SME(SM_BufferReverseBits(pRet)); // re-reverse bits to keep aligned for Public key.

   SME_FINISH_CATCH;
   return pRet;
}

//////////////////////////////////////////////////////////////////////////
CSM_Alg* CSM_CertificateChoice::GetPublicKeyAlg()
{
   CSM_Alg *pRet = NULL;

   SME_SETUP("CSM_CertificateChoice::GetPublicKeyAlg");

   // if m_pSnaccCert is null then the cert has not been decoded
   // and we must decode it before we can return the requested value
   if (m_pSnaccCert == NULL)
      SME(Decode());

   if ((pRet = new CSM_Alg(*(m_pSnaccCert->certificateToSign->
         subjectPublicKeyInfo->algorithm))) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

   SME_FINISH_CATCH;
   return pRet;
}

//////////////////////////////////////////////////////////////////////////
CSM_DN* CSM_CertificateChoice::GetIssuer()
{
   SME_SETUP("CSM_CertificateChoice::GetIssuer");

   // if m_pIssuer is null, then the Cert has not been decoded
   // and we must decode it before we can return the requested value
   if (m_pIssuer == NULL)
      SME(Decode());

   SME_FINISH_CATCH;
   return m_pIssuer;
}

//////////////////////////////////////////////////////////////////////////
CSM_DN* CSM_CertificateChoice::GetSubject()
{
   SME_SETUP("CSM_CertificateChoice::GetSubject");

   // if m_pSubject is null, then the Cert has not been decoded
   // and we must decode it before we can return the requested value
   if (m_pSubject == NULL)
      SME(Decode());

   SME_FINISH_CATCH;
   return m_pSubject;
}

//////////////////////////////////////////////////////////////////////////
CSM_Buffer* CSM_CertificateChoice::GetSerial()
{
   CSM_Buffer *pRet = NULL;

   SME_SETUP("CSM_CertificateChoice::GetSerial");

   // if m_pSnaccCert is null then the Cert has not been decoded
   // and we must decode it before we can return the requested value
   if (m_pSnaccCert == NULL)
      SME(Decode());

   if ((pRet = new CSM_Buffer(
         (char*)m_pSnaccCert->certificateToSign->serialNumber,
         m_pSnaccCert->certificateToSign->serialNumber.Len())) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

   SME_FINISH_CATCH;
   return pRet;
}

//////////////////////////////////////////////////////////////////////////
void CSM_CertificateChoice::SetSNACCCertChoices(CertificateChoices 
                                                &SNACCCertChoices)
{
   SME_SETUP("CSM_CertificateChoice::SetSNACCCertChoices");
   if (m_pCert)
      delete m_pCert;
   ENCODE_BUF(&SNACCCertChoices, m_pCert);
   if (SNACCCertChoices.choiceId == CertificateChoices::certificateCid)
   {
      if ((m_pSnaccCert = new Certificate) == NULL)
         SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
      *m_pSnaccCert = *SNACCCertChoices.certificate;
      m_lCertType = CSM_CertificateChoice::CERT_TYPE_CERT;
   }
   else if (SNACCCertChoices.choiceId == CertificateChoices::attrCertCid)
   {
      if ((m_pSnaccAttr = new AttributeCertificate) == NULL)
         SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
      *m_pSnaccAttr = *SNACCCertChoices.attrCert;
      m_lCertType = CSM_CertificateChoice::CERT_TYPE_ATTR_CERT;
   }
   else
      SME_THROW(SM_MEMORY_ERROR, 
            "Unsupported Certificate Choice(Extended)", NULL);

   SME_FINISH_CATCH;
}

// EOF sm_certChoice.cpp
