#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_StrFuncs.cpp 1.4 1/30/98 16:50:29"; 
#endif

#include <stdio.h>
#include <stdlib.h>
#include "sm_util.h"

/*
 *
 * This file contains functions that convert STRING (char *) data into 
 * different structures.
 *
 */

/*
 * FUNCTION: SM_Hex2Buffer()
 * 
 * PURPOSE: convert a string of HEX values into strings of characters.
 *          Store the result in a CSM_Buffer.
 *
 * NOTE: input must be null terminated.
 *
 * EXAMPLE:
 *         if input equals "000000" this function should return in output
 *         string of length 3 with with 3 NULL characters in the string.
 *
 * ERROR CONDITIONS:
 *         memory error
 *         input is not a multiple of 2.
 *         input is not in HEX format.
 */ 
SM_RET_VAL SM_Hex2BufferAlloc (char *pszHex, CSM_Buffer *&buffer)
{
   SME_SETUP("SM_Hex2Buffer Alloc");

   if ((buffer = new CSM_Buffer) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

   SM_Hex2Buffer(pszHex, buffer);

   SME_FINISH_CATCH

   return (SM_NO_ERROR);
}

// no alloc version
SM_RET_VAL SM_Hex2Buffer (char *pszHex, CSM_Buffer *buffer)
{
   SM_RET_VAL lError = SM_NO_ERROR;
   int  nInt; // int. which has an ascii val. of a character
   char *pchOut; // temp pointer to strStruct
   unsigned char *puchIn; // temp pointer to hex
   SM_SIZE_T lLength;

   SME_SETUP("SM_Hex2Buffer");

   if (buffer == NULL)
      SME_THROW(SM_MISSING_PARAM, NULL, NULL)

   // if this is hex it should divisible by two.   If not return an error.
   if (strlen(pszHex) % 2 != 0)
      lError = 1;
      
   if (!lError)
   {
      buffer->Open(SM_FOPEN_WRITE);
      lLength = strlen(pszHex) / 2;
      pchOut = buffer->Alloc(lLength);
      puchIn = (unsigned char *)pszHex;
      while (*puchIn != '\0' && !lError)
      {
         if (sscanf((char*)puchIn, "%2x", &nInt) > 0)
         {
            *pchOut = nInt;
            ++pchOut;
            puchIn += 2;
         }
         else
            lError = 1;
      }
      buffer->Flush();
      buffer->Close();
   }

   SME_FINISH_CATCH

   return (lError);
}

// EOF sm_StrFuncs.cpp
